# -*- coding: utf-8 -*-

# Copyright (c) 2002, 2003 Detlev Offenbach <detlev@die-offenbachs.de>
#

"""
Module implementing a dialog for editing breakpoints.
"""

from qt import *

from UI.Info import *
from BreakpointForm import BreakpointForm


class BreakpointDialog(BreakpointForm):
    """
    Class implementing a dialog for editing breakpoints.
    
    In this dialog all currently set breakpoint can be delete or be changed.
    Unconditional breakpoints can be converted to conditional ones by
    entering a condition term, conditional ones to unconditional by 
    removing the condition term. This dialog has three different modes.
    In 'project' mode all breakpoints of the current project are shown,
    in 'global' mode all breakpoints and in 'file' mode the breakpoints of
    a specific file.
    """
    def __init__(self,parent = None,name = None,modal = 0,fl = 0,fn = None):
        """
        Constructor
        
        Arguments
        
            parent -- parent widget of this dialog (QWidget)
            
            name -- the name of this dialog (string or QString)
            
            modal -- flag to indicate a modal dialog (boolean)
            
            fl -- window flags
            
            fn -- determines the mode of the dialog
            
                None -- the 'global' breakpoint dialog
                
                'project' -- the 'project' breakpoint dialog
                
                filename -- the 'file' breakpoint dialog
        """
        BreakpointForm.__init__(self,parent,name,modal,fl)

        if fn:
            if fn == 'project':
                self.setCaption(self.trUtf8('%1 - Project Breakpoints')
                    .arg(Program))
            else:
                self.setCaption(self.trUtf8('%1 - Breakpoints of %2')
                    .arg(Program)
                    .arg(fn))
        else:
            self.setCaption(self.trUtf8('%1 - All Breakpoints')
                .arg(Program))

        if not fn or fn == 'project':
            self.bpListview.addColumn(self.trUtf8("Filename"))
        self.bpListview.addColumn(self.trUtf8(" # "))
        self.bpListview.addColumn(self.trUtf8("Condition"))
        self.bpListview.setSelectionMode(QListView.Extended)
        self.bpListview.setSorting(0)
        
        self.filename = fn
        self.viewmanager = parent.getViewManager()
        
        # populate the listbox
        if not fn or fn == 'project':
            if fn is None:
                bpDict = self.parent().dbg.getAllBreakpoints()
            else:
                bpDict = self.parent().dbg.getProjectBreakpoints()
            for fname, bpList in bpDict.items():
                for (lineno, cond) in bpList:
                    QListViewItem(self.bpListview, fname, str(lineno), cond)
        else:
            bpList = self.parent().dbg.getFileBreakpoints(fn)
            for (lineno, cond) in bpList:
                QListViewItem(self.bpListview, str(lineno), cond)
                
        self.currItem = None
        self.condHistory = QStringList()
        
        # create the popup menu
        self.menu = QPopupMenu(self)
        self.menu.insertItem(self.trUtf8('Change condition...'), self.handleChangeCondition)
        self.menu.insertItem(self.trUtf8('Delete breakpoint'), self.handleDeleteSingle)

    def handleDeleteAll(self):
        """
        Private slot to delete all breakpoints listed in the listview.
        """
        item = self.bpListview.firstChild()
        
        while item:
            if not self.filename or self.filename == 'project':
                fn = str(item.text(0))
                line = int(str(item.text(1)))
            else:
                fn = str(self.filename)
                line = int(str(item.text(0)))
            self.viewmanager.handleBreakpoint(fn, line)
            self.bpListview.takeItem(item)
            del item
            item = self.bpListview.firstChild()

    def handleDeleteSelected(self):
        """
        Private slot to delete the selected breakpoints of the listview.
        """
        item = self.bpListview.firstChild()
        
        while item:
            if item.isSelected():
                if not self.filename or self.filename == 'project':
                    fn = str(item.text(0))
                    line = int(str(item.text(1)))
                else:
                    fn = str(self.filename)
                    line = int(str(item.text(0)))
                self.viewmanager.handleBreakpoint(fn, line)
                olditem = item
                item = olditem.itemBelow()
                self.bpListview.takeItem(olditem)
                del olditem
            else:
                item = item.itemBelow()

    def handleDeleteSingle(self):
        """
        Private slot to delete a single breakpoint.
        """
        if self.currItem:
            if not self.filename or self.filename == 'project':
                fn = str(self.currItem.text(0))
                line = int(str(self.currItem.text(1)))
            else:
                fn = str(self.filename)
                line = int(str(self.currItem.text(0)))
            # we simulate a button press in the breakpoint column 
            # of the source window
            self.viewmanager.handleBreakpoint(fn, line, Qt.LeftButton)
            self.bpListview.takeItem(self.currItem)
            del self.currItem
            self.currItem = None
        
    def handleChangeCondition(self):
        """
        Private slot to change the condition of the selected breakpoint.
        """
        if self.currItem:
            if not self.filename or self.filename == 'project':
                fn = str(self.currItem.text(0))
                line = int(str(self.currItem.text(1)))
                cond = self.currItem.text(2)
            else:
                fn = str(self.filename)
                line = int(str(self.currItem.text(0)))
                cond = self.currItem.text(1)
                
            if cond == None:
                cond = ''
            curr = self.condHistory.findIndex(cond)
            if curr == -1:
                self.condHistory.prepend(cond)
                curr = 0
                    
            cond, ok = QInputDialog.getItem(
                            self.trUtf8('Breakpoint'),
                            self.trUtf8('Enter condition for breakpoint'),
                            self.condHistory, curr, 1)
                                
            if not ok:
                self.currItem = None
                return
                
            # we simulate a button press in the breakpoint column 
            # of the source window
            self.viewmanager.handleBreakpoint(fn, line, Qt.LeftButton | Qt.ShiftButton, cond)
            
            if self.filename and self.filename != 'project':
                self.currItem.setText(1, cond)
            else:
                self.currItem.setText(2, cond)
            self.currItem = None
        
    def handleListContextmenu(self, item, pos, col):
        """
        Private slot to show the context menu of the listview.
        
        Arguments
        
            item -- the item under the mouse cursor (QListViewItem)
            
            pos -- global position of the context menu (QPoint)
            
            col -- column of the mouse cursor (int)
        """
        if item:
            self.currItem = item
            self.menu.popup(pos)
