# -*- coding: utf-8 -*-

# Copyright (c) 2002, 2003 Detlev Offenbach <detlev@die-offenbachs.de>
#

"""
Module implementing the variables viewer widget.
"""

import string
import types

from qt import QListView, QListViewItem, QWhatsThis, SIGNAL

from Config import ConfigVarTypeDispStrings, ConfigVarTypeStrings
from VariableDetailDialog import VariableDetailDialog


class VariablesViewer(QListView):
    """
    Class implementing the variables viewer widget.
    
    This widget is used to display the variables of the program being
    debugged in a tree. Compound types will be shown with
    their main entry first. Once the subtree has been expanded, the 
    individual entries will be shown. Double clicking an entry will
    popup a dialog showing the variables parameters in a more readable
    form. This is especially useful for lengthy strings.
    
    This widget has two modes for displaying the global and the local
    variables.
    
    """
    def __init__(self,parent=None,scope=1):
        """
        Constructor
        
        Arguments
        
            parent -- the parent (QWidget)
            
            scope -- flag indicating global (0) or local (1) variables
        """
        QListView.__init__(self,parent)
        
        self.indicators = {'list' : '[]', 'tuple' : '()', 'dict' : '{}'}
        
        self.setRootIsDecorated(1)
        self.setAllColumnsShowFocus(1)
        self.scope = scope
        if scope:
            self.setCaption(self.trUtf8("Global Variables"))
            self.addColumn(self.trUtf8("Globals"), 100)
            QWhatsThis.add(self,self.trUtf8(
                """<b>The Global Variables Viewer Window</b>"""
                """<p>This window displays the global variables"""
                """ of the debugged program.</p>"""
            ))
        else:
            self.setCaption(self.trUtf8("Local Variables"))
            self.addColumn(self.trUtf8("Locals"), 100)
            QWhatsThis.add(self,self.trUtf8(
                """<b>The Local Variables Viewer Window</b>"""
                """<p>This window displays the local variables"""
                """ of the debugged program.</p>"""
            ))
        
        self.addColumn(self.trUtf8("Value"), 150)
        self.addColumn(self.trUtf8("Type"))
        self.setSorting(0)
        
    def showVariables(self, vlist):
        """
        Public method to show variables in a listview.
        
        Arguments
        
            vlist -- the list of variables to be displayed. Each
              listentry is a tuple of three values.
              
                - the variable name (string)
                
                - the variables type (string)
                
                - the variables value (string)
        """
        self.clear()
        
        if len(vlist):
            itm = None
            for (var, vtype, value) in vlist:
                slist = string.split(var,'.',1)
                # format the type nicely
                editable = 0
                try:
                    i = ConfigVarTypeStrings.index(vtype)
                    dvtype = self.trUtf8(ConfigVarTypeDispStrings[i])
                except ValueError:
                    dvtype = vtype
                if len(slist) > 1:
                    item = self.findItem(slist[0], 0)
                    if item:
                        var = slist[1]
                else:
                    item = None
                itm = self.addItem(item, vtype, var, value, dvtype)

    def addItem(self, parent, vtype, var, value, dvtype):
        """
        Private method used to add an item to the listview.
        
        If the item is of a type with subelements (i.e. list, dictionary, 
        tuple), these subelements are added by calling this method recursively.
        
        Arguments
        
            parent -- the parent of the item to be added
            (QListViewItem or None)
            
            vtype -- the type of the item to be added
            (string)
            
            var -- the variable name (string)
            
            value -- the value string (string)
            
            dvtype -- the typestring to be displayed
            (QString or string)
            
        Returns
        
            The item that was added to the listview (QListViewItem).
        """
        if parent is None:
            parent = self
        try:
            dvar = '%s%s' % (var, self.indicators[vtype])
        except:
            dvar = var
            
        if vtype in ['list', 'tuple']:
            itm = QListViewItem(parent, dvar, '', dvtype)
            try:
                try:
                    list = eval(value)
                except SyntaxError:
                    list = eval(value.replace('<','"<').replace('>','>"'))
                    
                for i in range(len(list)):
                    lvtype = self.getType(list[i])
                    ldvtype = self.getDispType(lvtype)
                    if lvtype in ['list', 'tuple', 'dict']:
                        self.addItem(itm, lvtype, str(i), str(list[i]), ldvtype)
                    else:
                        QListViewItem(itm, str(i), str(list[i]), ldvtype)
            except:
                pass
                
        elif vtype == 'dict':
            itm = QListViewItem(parent, dvar, '', dvtype)
            try:
                try:
                    dict = eval(value)
                except SyntaxError:
                    dict = eval(value.replace('<','"<').replace('>','>"'))
                    
                for key, val in dict.items():
                    dvtype = self.getType(val)
                    ddvtype = self.getDispType(dvtype)
                    if dvtype in ['list', 'tuple', 'dict']:
                        self.addItem(itm, dvtype, str(key), str(val), ddvtype)
                    else:
                        QListViewItem(itm, str(key), str(val), ddvtype)
            except:
                pass
                
        else:
            itm = QListViewItem(parent, dvar, value, dvtype)
            
        return itm

    def getType(self, value):
        """
        Private method used to get the type of the value passed in.
        
        Arguments
        
            value -- the real value (any)
            
        Returns
        
            type (string).
        """
        if type(value) is types.StringType and \
           value.startswith('<'):
            if value.find('instance') > -1:
                return 'instance'
            elif value.find('object') > -1:
                val = value.split()[0][1:]
                return val
            
        valtypestr = str(type(value))[1:-1]
        if string.split(valtypestr,' ',1)[0] == 'class':
            # handle new class type of python 2.2+
            valtype = valtypestr[7:-1]
        else:
            valtype = valtypestr[6:-1]
        return valtype

    def getDispType(self, vtype):
        """
        Private method used to get the display string for type vtype.
        
        Arguments
        
            vtype -- the type, the display string should be looked up for
              (string)
            
        Returns
        
            displaystring (string or QString)
        """
        try:
            i = ConfigVarTypeStrings.index(vtype)
            dvtype = self.trUtf8(ConfigVarTypeDispStrings[i])
        except ValueError:
            dvtype = vtype
        return dvtype

    def contentsMouseDoubleClickEvent(self, mouseEvent):
        """
        Protected method of QListView. 
        
        Reimplemented to disable expanding/collapsing
        of items when double-clicking. Instead the double-clicked entry is opened.
        """
        itm = self.currentItem()
        
        val = itm.text(1)
        
        if val.isEmpty():
            return  # do not display anything, if the variable has no value
            
        vtype = itm.text(2)
        name = str(itm.text(0))
        
        par = itm.parent()
        nlist = [name]
        # build up the fully qualified name
        while par is not None:
            pname = str(par.text(0))
            if pname[-2:] in ['[]', '{}', '()']:
                nlist[0] = '[%s]' % nlist[0]
                nlist.insert(0, pname[:-2])
            else:
                nlist.insert(0, '%s.' % pname)
            par = par.parent()
            
        name = string.join(nlist, '')
        # now show the dialog
        dlg = VariableDetailDialog(name, vtype, val)
        dlg.exec_loop()
