# -*- coding: utf-8 -*-

# Copyright (c) 2002, 2003 Detlev Offenbach <detlev@die-offenbachs.de>
#

"""
Module implementing the project browser part of the eric3 UI.
"""

import os
import sys
import shutil

from qt import *

from UI.Browser import *
from Icons.FileIcons import IconDirClosed, IconDirOpen

class ProjectBrowserSimpleDir(BrowserNode):
    """
    Class implementing a small wrapper around BrowserNode.
    
    It sets a pixmap depending on the open state.
    """
    def __init__(self,parent,text):
        """
        Constructor
        
        Arguments
        
            parent -- parent widget of this item (QListView or QListViewItem)
            
            text -- text to be displayed (string or QString)
        """
        BrowserNode.__init__(self, parent, text, None)
        
        self.setExpandable(1)
        self.setOpen(1)
        
    def setOpen(self, o):
        """
        Slot called to open/close the tree node.
        
        It sets the pixmap displayed next to the tree node depending of the
        open status of the node.
        
        Arguments
        
            o -- flag indicating the open status (boolean)
        """
        if o:
            self.setPixmap(0, IconDirOpen)
        else:
            self.setPixmap(0, IconDirClosed)
            
        QListViewItem.setOpen(self,o)
        
class ProjectBrowserDirectory(BrowserDirectory):
    """
    Class implementig a BrowserNode that represents a directory.  
    """
    def __init__(self,parent,dinfo,after,full=1,bold=0,vcs=None):
        """
        Constructor
        
        Arguments
        
            parent -- the parent Browser or BrowserNode
            
            dinfo -- name of the directory (string or QString)
            
            after -- the sibling this node is position after or None, if this is
                the first child
            
            full -- flag indicating whether the full pathname of the directory
                should be displayed
            
            bold -- flag indicating whether the directory name should be displayed
                in bold
            
            vcs -- if a vcs object is given, the VCS status of the files
                contained in the directory is displayed
        """
        BrowserDirectory.__init__(self,parent,dinfo,after,full,bold)
        
        self.vcs = vcs
        
    def setOpen(self,o):
        """
        Slot called to open/close the directory node.
        
        Arguments
        
            o -- flag indicating the open status (boolean)
        """
        if o:
            self.dir = QDir(self.dirName)
            last = None

            for f in self.dir.entryInfoList():
                if str(f.fileName()) in ('.', '..'):
                    continue

                if f.isDir():
                    node = ProjectBrowserDirectory(self,
                        str(QDir.convertSeparators(f.absFilePath())),last,0,0,self.vcs)
                else:
                    node = BrowserFile(self,
                        str(QDir.convertSeparators(f.absFilePath())),last)
                if self.vcs is not None:
                    state = self.vcs.vcsRegisteredState(f.absFilePath())
                    if state == self.vcs.canBeCommitted:
                        node.setText(1, self.vcs.vcsName())
                    else:
                        node.setText(1, 
                            qApp.translate("ProjectBrowserDirectory", "local"))

                last = node
                self.children.append(node)

            self.setPixmap(0,IconDirOpen)
        else:
            for child in self.children:
                self.takeItem(child)
                del child
            self.children = []
                
            self.setPixmap(0,IconDirClosed)

        QListViewItem.setOpen(self,o)


class ProjectBrowser(QTabWidget):
    """
    Class implementing the project browser part of the eric3 UI.
    
    It generates a widget with four tabs. The individual tabs contain
    the project sources browser, the project forms browser,
    the project translations browser and a browser for stuff
    that doesn't fit these categories.
    
    Signals
    
        editorSaved(string) -- emitted after an editor has been saved
    """
    def __init__(self,project,qtdir,parent=None):
        """
        Constructor
        
        Arguments
        
            project -- reference to the project object
            
            qtdir -- Qt installation directory (string)
            
            parent -- parent widget (QWidget)
        """
        QTabWidget.__init__(self, parent)
        self.project = project
        
        # add the sources browser
        self.psBrowser = ProjectSourcesBrowser(self.project, self)
        self.addTab(self.psBrowser, self.psBrowser.caption())
        self.connect(self.project, PYSIGNAL('projectClosed'),
                self.psBrowser.handleProjectClosed)
        self.connect(self.project, PYSIGNAL('projectOpened'),
                self.psBrowser.handleProjectOpened)
        self.connect(self.project, PYSIGNAL('newProject'),
                self.psBrowser.handleNewProject)
        self.connect(self.project, PYSIGNAL('projectSourceAdded'),
                self.psBrowser.handleProjectSourceAdded)
        self.connect(self, PYSIGNAL('editorSaved'),
                self.psBrowser.handleEditorSaved)
        
        # add the forms browser
        self.pfBrowser = ProjectFormsBrowser(self.project, qtdir, self)
        self.addTab(self.pfBrowser, self.pfBrowser.caption())
        self.connect(self.project, PYSIGNAL('projectClosed'),
                self.pfBrowser.handleProjectClosed)
        self.connect(self.project, PYSIGNAL('projectOpened'),
                self.pfBrowser.handleProjectOpened)
        self.connect(self.project, PYSIGNAL('newProject'),
                self.pfBrowser.handleNewProject)
        self.connect(self.project, PYSIGNAL('projectFormAdded'),
                self.pfBrowser.handleProjectFormAdded)
        self.connect(self.pfBrowser, PYSIGNAL('projectSourceAdded'),
                self.psBrowser.handleProjectSourceAdded)
        
        # add the translations browser
        self.ptBrowser = ProjectTranslationsBrowser(self.project, qtdir, self)
        self.addTab(self.ptBrowser, self.ptBrowser.caption())
        self.connect(self.project, PYSIGNAL('projectClosed'),
                self.ptBrowser.handleProjectClosed)
        self.connect(self.project, PYSIGNAL('projectOpened'),
                self.ptBrowser.handleProjectOpened)
        self.connect(self.project, PYSIGNAL('newProject'),
                self.ptBrowser.handleNewProject)
        self.connect(self.project, PYSIGNAL('projectLanguageAdded'),
                self.ptBrowser.handleProjectLanguageAdded)
        
        # add the others browser
        self.poBrowser = ProjectOthersBrowser(self.project, self)
        self.addTab(self.poBrowser, self.poBrowser.caption())
        self.connect(self.project, PYSIGNAL('projectClosed'),
                self.poBrowser.handleProjectClosed)
        self.connect(self.project, PYSIGNAL('projectOpened'),
                self.poBrowser.handleProjectOpened)
        self.connect(self.project, PYSIGNAL('newProject'),
                self.poBrowser.handleNewProject)
        self.connect(self.project, PYSIGNAL('projectOthersAdded'),
                self.poBrowser.addNode)
        
        # add signal connection to ourself
        self.connect(self.project, PYSIGNAL('projectOpened'),
                self.handleProjectOpened)
                
        self.showPage(self.psBrowser)
        
    def handleProjectOpened(self):
        """
        Private slot to handle the projectOpened signal.
        """
        self.showPage(self.psBrowser)
        
    def handleEditorSaved(self, fn):
        """
        Public slot to handle the editorSaved signal.
        
        It simply reemits it.
        """
        self.emit(PYSIGNAL('editorSaved'), (fn,))
        
class PBrowser(Browser):
    """
    Baseclass implementing common functionality for the various browsers.
    """
    def __init__(self,project,pdataKey,parent=None):
        """
        Constructor
        
        Arguments
        
            project -- reference to the project object
            
            pdataKey -- key of the filelist the browser object is handling (string)
            
            parent -- parent widget of this browser
        """
        QListView.__init__(self,parent)
    
        self.pdataKey = pdataKey
        self.project = project
        self.setRootIsDecorated(1)
        self.setSorting(0)
        self.addColumn(self.trUtf8('Name'))
        self.addColumn(self.trUtf8('VCS Status'))
        
        self.children = []
        self.projectOpened = 0
        
        self.populateTree()
        
        self.connect(self,SIGNAL('contextMenuRequested(QListViewItem *, const QPoint &, int)'),
                     self.handleContextMenu)

        req = QSize(250,350)

        if parent is not None:
            req = req.boundedTo(parent.size())

        self.resize(req)

        self.createPopupMenus()
        
    def createPopupMenus(self):
        """
        Private overloaded method to generate the popup menu.
        """
        Browser.createPopupMenus(self)
        self.menuItems = []
        
    def handleNewProject(self):
        """
        Private slot to handle the newProject signal.
        """
        if self.backMenu is not None:
            self.backMenu.setEnabled(1)
        self.projectOpened = 1
        
    def handleProjectClosed(self):
        """
        Private slot to handle the projectClosed signal.
        """
        self.children = []
        self.clear()
        if self.backMenu is not None:
            self.backMenu.setEnabled(0)
        self.projectOpened = 0
        
    def handleProjectOpened(self):
        """
        Private slot to handle the projectOpened signal.
        """
        self.populateTree()
        
        if self.backMenu is not None:
            self.backMenu.setEnabled(1)
        self.projectOpened = 1
        
    def populateTree(self):
        """
        Private method used to populate the listview.
        """
        if self.project.vcs is not None:
            states = {}
            for fn in self.project.pdata[self.pdataKey]:
                states[os.path.normcase(os.path.join(self.project.ppath, fn))] = 0
            if self.pdataKey == "OTHERS":
                for dir in self.project.otherssubdirs:
                    if not os.path.isabs(dir):
                        dir = os.path.join(self.project.ppath, dir)
                    states = self.project.vcs.vcsAllRegisteredStates(states, dir)
            else:
                for dir in self.project.subdirs:
                    states = self.project.vcs.vcsAllRegisteredStates(states, 
                        os.path.join(self.project.ppath, dir))
            
        # Show the entry in bold in the others browser to make it more distinguishable
        if self.pdataKey == "OTHERS":
            bold = 1
        else:
            bold = 0
            
        for fn in self.project.pdata[self.pdataKey]:
            fname = os.path.join(self.project.ppath, fn)
            if self.pdataKey == "TRANSLATIONS":
                dt, ext = os.path.splitext(fn)
                dt = dt.split('_')[-1]
                node = BrowserFile(self, fname, None, 1, dt)
            else:
                parent, dt = self.findParentNode(fn)
                if os.path.isdir(fname):
                    node = ProjectBrowserDirectory(parent, fname, None, 0, 
                        bold, self.project.vcs)
                else:
                    node = BrowserFile(parent, fname, None, 1, dt, bold)
            self.children.append(node)
            if self.project.vcs is not None:
                if states[os.path.normcase(fname)] == self.project.vcs.canBeCommitted:
                    node.setText(1, self.project.vcs.vcsName())
                else:
                    node.setText(1, self.trUtf8("local"))
                    
    def findItem(self, text, column, node=None):
        """
        Reimplemented method
        
        It is used to find a specific item with text in column,
        that is a child of node. If node is None, a child of the
        QListView is searched.
        
        Arguments
        
            text -- text to search for (string or QString)
            
            column -- index of column to search in (int)
            
            node -- start point of the search
            
        Returns
        
            the found item
        """
        if node is None:
            node = self
            
        itm = node.firstChild()
        while itm is not None:
            if QString.compare(itm.text(column), text) == 0:
                break
            itm = itm.nextSibling()
        return itm
        
    def findParentNode(self, fn):
        """
        Private method used to find or create the parent node.
        
        Arguments
        
            fn -- filename to use for the search
            
        Returns
        
            tuple of two values giving the parent node and the shortened filename
        """
        pathlist = QStringList.split(QRegExp(r'/|\\'), fn)
        if len(pathlist) > 1:
            oldnode = self
            for p in pathlist[:-1]:
                node = self.findItem(p, 0, oldnode)
                if node is None:
                    node = ProjectBrowserSimpleDir(oldnode, p)
                oldnode = node
            return (node, QString(pathlist[-1]))
        else:
            return (self, fn)
            
    def removeNode(self, node):
        """
        Private method to remove a parent (dir) node, if it doesn't have any children.
        
        Arguments
        
            node -- node to remove
        """
        if node is None:
            return
            
        parent = node.parent()
        if parent is None:
            self.takeItem(node)
        else:
            parent.takeItem(node)
            if parent.childCount() == 0:
                self.removeNode(parent)
                del parent
        
    def nodeAdded(self, node, name):
        """
        Public method used to perform common operations on a new node.
        
        Arguments
        
            node -- node to work on
            
            name -- filename belonging to this node
        """
        self.updateVCSStatus(node, name)
        
    def updateVCSStatus(self, node, name):
        """
        Private method used to set the vcs status of a node.
        
        Arguments
        
            node -- node to work on
            
            name -- filename belonging to this node
        """
        if self.project.vcs is not None:
            state = self.project.vcs.vcsRegisteredState(name)
            if state == self.project.vcs.canBeCommitted:
                node.setText(1, self.project.vcs.vcsName())
            else:
                node.setText(1, self.trUtf8("local"))
        
    def addVCSMenu(self, menu):
        """
        Public method used to add the VCS menu to all project browsers.
        
        Arguments
        
            menu -- reference to the menu to be amended
        """
        menu.insertSeparator()
        self.vcsMenuItems = []
        itm = menu.insertItem(self.trUtf8('Update from repository'), self.handleVCSUpdate)
        self.vcsMenuItems.append(itm)
        itm = menu.insertItem(self.trUtf8('Commit changes to repository...'), self.handleVCSCommit)
        self.vcsMenuItems.append(itm)
        self.vcsMenuAdd = menu.insertItem(self.trUtf8('Add to repository'), self.handleVCSAdd)
        itm = menu.insertItem(self.trUtf8('Remove from repository (and disk)'), self.handleVCSRemove)
        self.vcsMenuItems.append(itm)
        itm = menu.insertItem(self.trUtf8('Show log'), self.handleVCSLog)
        self.vcsMenuItems.append(itm)
        itm = menu.insertItem(self.trUtf8('Show difference to repository'), self.handleVCSDiff)
        self.vcsMenuItems.append(itm)
        itm = menu.insertItem(self.trUtf8('Show history'), self.handleVCSHistory)
        self.vcsMenuItems.append(itm)
        itm = menu.insertItem(self.trUtf8('Show status'), self.handleVCSStatus)
        self.vcsMenuItems.append(itm)
        
    def handleShowPopupMenu(self, menu):
        """
        Slot called before the context menu is shown. 
        
        It enables/disables the VCS menu entries depending on the overall 
        VCS status and the file status.
        
        Arguments
        
            menu -- reference to the menu to be shown
        """
        if self.project.vcs is None:
            for itm in self.vcsMenuItems:
                menu.setItemEnabled(itm, 0)
            menu.setItemEnabled(self.vcsMenuAdd, 0)
            for itm in self.menuItems:
                menu.setItemEnabled(itm, 1)
        else:
            if str(self.currentItem().text(1)) == self.project.vcs.vcsName():
                for itm in self.vcsMenuItems:
                    menu.setItemEnabled(itm, 1)
                menu.setItemEnabled(self.vcsMenuAdd, 0)
                for itm in self.menuItems:
                    menu.setItemEnabled(itm, 0)
            else:
                for itm in self.vcsMenuItems:
                    menu.setItemEnabled(itm, 0)
                menu.setItemEnabled(self.vcsMenuAdd, 1)
                for itm in self.menuItems:
                    menu.setItemEnabled(itm, 1)
        
    def handleVCSUpdate(self):
        """
        Private slot called by the context menu to add a file to the VCS repository.
        """
        itm = self.currentItem()
        fn = str(itm.fileName())
        self.project.vcs.vcsUpdate(fn)
        
    def handleVCSCommit(self):
        """
        Private slot called by the context menu to commit the changes to the VCS repository.
        """
        itm = self.currentItem()
        fn = str(itm.fileName())
        self.project.vcs.vcsCommit(fn, '')
        
    def handleVCSAdd(self):
        """
        Private slot called by the context menu to add the selected file to the VCS repository.
        """
        itm = self.currentItem()
        fn = str(itm.fileName())
        self.project.vcs.vcsAdd(fn)
        self.updateVCSStatus(itm, fn)
        
    def handleVCSRemove(self):
        """
        Private slot called by the context menu to remove the selected file from the VCS repository.
        """
        itm = self.currentItem()
        fn = str(itm.fileName())
        res = QMessageBox.warning(None,
            self.trUtf8("Remove file from repository"),
            self.trUtf8("Dou you really want to remove <b>%1</b> from the repository (and disk)?")
                .arg(fn),
            self.trUtf8("&Yes"), self.trUtf8("&No"), None, 1)
        if res == 0:
            self.project.vcs.vcsRemove(fn)
            if not os.path.exists(fn):
                self.handleRemove() # remove file from project
        
    def handleVCSLog(self):
        """
        Private slot called by the context menu to show the VCS log of a file.
        """
        itm = self.currentItem()
        fn = str(itm.fileName())
        self.project.vcs.vcsLog(fn)
        
    def handleVCSDiff(self):
        """
        Private slot called by the context menu to show the difference of a file to the repository.
        """
        itm = self.currentItem()
        fn = str(itm.fileName())
        self.project.vcs.vcsDiff(fn)
        
    def handleVCSHistory(self):
        """
        Private slot called by the context menu to show the history of a file.
        """
        itm = self.currentItem()
        fn = str(itm.fileName())
        self.project.vcs.vcsHistory(fn)
        
    def handleVCSStatus(self):
        """
        Private slot called by the context menu to show the status of a file.
        """
        itm = self.currentItem()
        fn = str(itm.fileName())
        self.project.vcs.vcsStatus(fn)

class ProjectSourcesBrowser(PBrowser):
    """
    A class used to display the Sources part of the project. 
    
    Via the context menu that
    is displayed by a right click the user can select various actions on
    the selected file.
    
    Signals
    
        closeSourceWindow(string) -- emitted after a file has been removed/deleted 
            from the project
    """
    def __init__(self,project,parent=None):
        """
        Constructor
        
        Arguments
        
            project -- reference to the project object
            
            parent -- parent widget of this browser (QWidget)
        """
        PBrowser.__init__(self,project,"SOURCES",parent)
    
        self.setCaption(self.trUtf8('Sources'))

        QWhatsThis.add(self,self.trUtf8(
            """<b>Project Sources Browser</b>"""
            """<p>This allows to easily see all sources contained in the current"""
            """ project. Several actions can be executed via the context menu.</p>"""
        ))
        
    def createPopupMenus(self):
        """
        Private overloaded method to generate the popup menu.
        """
        PBrowser.createPopupMenus(self)
        
        self.pyMenu.insertSeparator()
        itm = self.pyMenu.insertItem(self.trUtf8('Remove from project'), self.handleRemove)
        self.menuItems.append(itm)
        itm = self.pyMenu.insertItem(self.trUtf8('Delete'), self.handleDelete)
        self.menuItems.append(itm)
        self.pyMenu.insertSeparator()
        self.pyMenu.insertItem(self.trUtf8('Add file...'), self.project.addPyFile)
        self.pyMenu.insertItem(self.trUtf8('Add python directory...'), self.project.addPyDir)
        self.addVCSMenu(self.pyMenu)

        self.backMenu = QPopupMenu(self)
        self.backMenu.insertItem(self.trUtf8('Add python file...'), self.project.addPyFile)
        self.backMenu.insertItem(self.trUtf8('Add python directory...'), self.project.addPyDir)
        self.backMenu.setEnabled(0)
        
        self.connect(self.pyMenu,SIGNAL('aboutToShow()'),self.handlePopupMenu)
        
    def handlePopupMenu(self):
        """
        Private slot called by the pyMenu aboutToShow signal.
        """
        self.handleShowPopupMenu(self.pyMenu)
        
    def handleRemove(self):
        """
        Private method to remove a file from the project.
        """
        itm = self.currentItem()
        fn2 = str(itm.fileName())
        fn = fn2.replace(self.project.ppath+os.sep, '')
        self.removeNode(itm)
        self.children.remove(itm)
        del itm
        
        self.project.removeFile(fn)
        self.emit(PYSIGNAL('closeSourceWindow'), (fn2,))
        
    def handleDelete(self):
        """
        Private method to delete a file from the project.
        """
        itm = self.currentItem()
        fn2 = str(itm.fileName())
        fn = fn2.replace(self.project.ppath+os.sep, '')
        res = QMessageBox.warning(self.parent(),
            self.trUtf8("Delete file"),
            self.trUtf8("Dou you really want to delete <b>%1</b> from the project?")
                .arg(fn),
            self.trUtf8("&Yes"), self.trUtf8("&No"), None, 1)
        if res == 0:
            self.emit(PYSIGNAL('closeSourceWindow'), (fn2,))
            if self.project.deleteFile(fn):
                self.removeNode(itm)
                self.children.remove(itm)
                del itm
    
    def handleProjectSourceAdded(self, fn):
        """
        Private slot to handle the projectSourceAdded signal.
        
        Arguments
        
            fn -- filename of the file that was added (string)
        """
        fname = os.path.join(self.project.ppath, fn)
        parent, dt = self.findParentNode(fn)
        node = BrowserFile(parent, fname, None, 1, dt)
        self.children.append(node)
        self.nodeAdded(node, fname)
        
    def handleEditorSaved(self, fn):
        """
        Public slot to handle the editorSaved signal.
        
        Arguments
        
            fn -- filename of the file that was saved
        """
        if not self.projectOpened:
            return
            
        list = QStringList.split(QRegExp(r'/|\\'), fn)
        newfn = QString(list[-1])
        itm = None
        for l in list:
            itm = self.findItem(l, 0, itm)
            
        if itm is not None:
            if itm.isOpen():
                # make it reread the info by closing it and reopen it
                openItems = self.getOpenChildren(itm)
                itm.setOpen(0)
                qApp.processEvents()
                itm.setOpen(1)
                if len(openItems):
                    for oi in openItems:
                        self.setOpenChildren(itm, oi)
                
    def getOpenChildren(self, itm):
        """
        Private method to get a list of open siblings of QListViewItem itm.
        
        Returns
        
            list of open siblings
        """
        olist = []
        child = itm.firstChild()
        while child is not None:
            if child.isOpen():
                olist.append(child.text(0))
                if child.childCount():
                    olist += self.getOpenChildren(child)
            child = child.nextSibling()
        return olist
        
    def setOpenChildren(self, itm, childstring):
        """
        Private method to find a child of a node and open it.
        
        Arguments
        
            itm -- the node to check
            
            childstring -- displaytext to search for (QString)
            
        Returns
        
            flag indicating success
        """
        child = itm.firstChild()
        while child is not None:
            if child.isOpen() and child.childCount():
                s = self.setOpenChildren(child, childstring)
                if s:
                    return 1
            if child.text(0).compare(childstring) == 0:
                child.setOpen(1)
                return 1
            child = child.nextSibling()
                
        return 0
        
class ProjectFormsBrowser(PBrowser):
    """
    A class used to display the forms part of the project. 
    
    Via the context menu that
    is displayed by a right click the user can select various actions on
    the selected file.
    
    Signals
    
        appendStdout(string) -- emitted after something was received from
            a QProcess on stdout
            
        appendStderr(string) -- emitted after something was received from
            a QProcess on stderr
            
        projectSourceAdded(string) -- emitted after a compile finished successfully
    """
    def __init__(self,project,qtdir,parent=None):
        """
        Constructor
        
        Arguments
        
            project -- reference to the project object
            
            qtdir -- path of the Qt installation directory (string)
            
            parent -- parent widget of this browser (QWidget)
        """
        self.qtdir = qtdir
        PBrowser.__init__(self,project,"FORMS",parent)
    
        self.setCaption(self.trUtf8('Forms'))
        
        QWhatsThis.add(self,self.trUtf8(
            """<b>Project Forms Browser</b>"""
            """<p>This allows to easily see all forms contained in the current"""
            """ project. Several actions can be executed via the context menu.</p>"""
        ))
        
    def createPopupMenus(self):
        """
        Private overloaded method to generate the popup menu.
        """
        self.menuItems = []
        self.menu = QPopupMenu(self)
        itm1 = self.menu.insertItem(self.trUtf8('Compile form'), self.handleCompile)
        itm2 = self.menu.insertItem(self.trUtf8('Compile all forms'), self.handleCompileAll)
        itm3 = self.menu.insertItem(self.trUtf8('Open in Qt-Designer'), self.handleOpen)
        self.menu.insertSeparator()
        itm = self.menu.insertItem(self.trUtf8('Remove from project'), self.handleRemove)
        self.menuItems.append(itm)
        itm = self.menu.insertItem(self.trUtf8('Delete'), self.handleDelete)
        self.menuItems.append(itm)
        self.menu.insertSeparator()
        self.menu.insertItem(self.trUtf8('Add form...'), self.project.addUiFile)
        self.menu.insertItem(self.trUtf8('Add forms directory...'), self.project.addUiDir)
        self.addVCSMenu(self.menu)

        self.backMenu = QPopupMenu(self)
        itm4 = self.backMenu.insertItem(self.trUtf8('Compile all forms'), self.handleCompileAll)
        self.backMenu.insertSeparator()
        self.backMenu.insertItem(self.trUtf8('Add form...'), self.project.addUiFile)
        self.backMenu.insertItem(self.trUtf8('Add forms directory...'), self.project.addUiDir)
        self.backMenu.setEnabled(0)

        if self.qtdir == None:
            self.menu.setItemEnabled(itm1, 0)
            self.menu.setItemEnabled(itm2, 0)
            self.menu.setItemEnabled(itm3, 0)
            self.backMenu.setItemEnabled(itm4, 0)
            
        self.connect(self.menu,SIGNAL('aboutToShow()'),self.handlePopupMenu)
        
    def handlePopupMenu(self):
        """
        Private slot called by the pyMenu aboutToShow signal.
        """
        self.handleShowPopupMenu(self.menu)
        
    def handleRemove(self):
        """
        Private method to remove a file from the project.
        """
        itm = self.currentItem()
        fn2 = str(itm.fileName())
        fn = fn2.replace(self.project.ppath+os.sep, '')
        self.removeNode(itm)
        self.children.remove(itm)
        del itm
        
        self.project.removeFile(fn)
        
    def handleDelete(self):
        """
        Private method to delete a file from the project.
        """
        itm = self.currentItem()
        fn2 = str(itm.fileName())
        fn = fn2.replace(self.project.ppath+os.sep, '')
        res = QMessageBox.warning(self.parent(),
            self.trUtf8("Delete form"),
            self.trUtf8("Dou you really want to delete <b>%1</b> from the project?")
                .arg(fn),
            self.trUtf8("&Yes"), self.trUtf8("&No"), None, 1)
        if res == 0:
            if self.project.deleteFile(fn):
                self.removeNode(itm)
                self.children.remove(itm)
                del itm
    
    def handleStdout(self):
        """
        Private slot to handle the readyReadStdout signal of the pyuic process.
        """
        while self.compileProc.canReadLineStdout():
            s = QString('pyuic: ')
            s.append(self.compileProc.readLineStdout())
            self.emit(PYSIGNAL('appendStdout'), (s,))
        
    def handleStderr(self):
        """
        Private slot to handle the readyReadStderr signal of the pyuic process.
        """
        while self.compileProc.canReadLineStderr():
            s = QString('pyuic: ')
            s.append(self.compileProc.readLineStderr())
            self.emit(PYSIGNAL('appendStderr'), (s,))
        
    def handleCompileUIDone(self):
        """
        Private slot to handle the processExit signal of the pyuic process.
        """
        self.compileRunning = 0
        if self.compileProc.normalExit():
            if self.compiledFile not in self.project.pdata["SOURCES"]:
                self.project.pdata["SOURCES"].append(self.compiledFile)
                self.emit(PYSIGNAL('projectSourceAdded'), (self.compiledFile,))
                self.project.setDirty(1)
        if not self.noDialog:
            QMessageBox.information(None,
                self.trUtf8("Form Compilation"),
                self.trUtf8("The compilation of the form file was successful."),
                QMessageBox.Ok)
        self.compileProc = None
                
    def handleCompileUI(self, fn, noDialog = 0, progress = None):
        """
        Privat method to compile a .ui file to a .py file.
        
        Arguments
        
            fn -- filename of the .ui file to be compiled
            
            noDialog -- flag indicating silent operations
            
            progress -- reference to the progress dialog
            
        Returns
        
            reference to the compile process (QProcess)
        """
        self.compileProc = QProcess(self)
        
        pyuic = 'pyuic'
        if sys.platform == "win32":
            pyuic = pyuic + '.exe'
        self.compileProc.addArgument(pyuic)
        
        (ofn, ext) = os.path.splitext(fn)
        self.compiledFile = ofn + '.py'
        fn = os.path.join(self.project.ppath, fn)
        ofn = os.path.join(self.project.ppath, self.compiledFile)
        
        self.compileProc.addArgument('-x')
        self.compileProc.addArgument('-o')
        self.compileProc.addArgument(ofn)
        self.compileProc.addArgument(fn)
        self.connect(self.compileProc, SIGNAL('processExited()'), self.handleCompileUIDone)
        self.connect(self.compileProc, SIGNAL('readyReadStdout()'), self.handleStdout)
        self.connect(self.compileProc, SIGNAL('readyReadStderr()'), self.handleStderr)
        
        self.compileRunning = 1
        self.noDialog = noDialog
        if self.compileProc.start():
            return self.compileProc
        else:
            if progress is not None:
                progress.cancel()
            QMessageBox.critical(self,
                self.trUtf8('Process Generation Error'),
                self.trUtf8(
                    'Could not start pyuic.<br>'
                    'Ensure that it is in the search path.'
                ),
                self.trUtf8('OK'))
            return None
        
    def handleCompile(self):
        """
        Private method to compile a form to a python file.
        """
        itm = self.currentItem()
        fn2 = str(itm.fileName())
        fn = fn2.replace(self.project.ppath+os.sep, '')
        self.handleCompileUI(fn)
        
    def handleCompileAll(self):
        """
        Private method to compile all forms to python files.
        """
        numForms = len(self.project.pdata["FORMS"])
        progress = QProgressDialog(self.trUtf8("Compiling forms..."), 
            self.trUtf8("Abort"), numForms, self, "progress", 1)
        progress.setMinimumDuration(0)
        i = 0
        
        for fn in self.project.pdata["FORMS"]:
            progress.setProgress(i)
            if progress.wasCancelled():
                break
            proc = self.handleCompileUI(fn, 1, progress)
            if proc is not None:
                while proc.isRunning():
                    qApp.processEvents()
                    QThread.msleep(300)
                    qApp.processEvents()
            else:
                break
            i += 1
            
        progress.setProgress(numForms)
        
    def handleProjectFormAdded(self, fn):
        """
        Private slot to handle the projectSourceAdded signal.
        """
        fname = os.path.join(self.project.ppath, fn)
        parent, dt = self.findParentNode(fn)
        node = BrowserFile(parent, fname, None, 1, dt)
        self.children.append(node)
        self.nodeAdded(node, fname)
        
class ProjectTranslationsBrowser(PBrowser):
    """
    A class used to display the translations part of the project. 
    
    Via the context 
    menu that is displayed by a right click the user can select various actions on
    the selected file.
    
    Signals
    
        appendStdout(string) -- emitted after something was received from
            a QProcess on stdout
            
        appendStderr(string) -- emitted after something was received from
            a QProcess on stderr
    """
    def __init__(self,project,qtdir,parent=None):
        """
        Constructor
        
        Arguments
        
            project -- reference to the project object
            
            qtdir -- path of the Qt installation directory (string)
            
            parent -- parent widget of this browser (QWidget)
        """
        self.qtdir = qtdir
        PBrowser.__init__(self,project,"TRANSLATIONS",parent)
    
        self.setCaption(self.trUtf8('Translations'))

        QWhatsThis.add(self,self.trUtf8(
            """<b>Project Translations Browser</b>"""
            """<p>This allows to easily see all translations contained in the current"""
            """ project. Several actions can be executed via the context menu.</p>"""
        ))
        
        self.lreleaseProc = None
        self.pylupdateProc = None
        
    def createPopupMenus(self):
        """
        Private overloaded method to generate the popup menu.
        """
        self.menuItems = []
        self.menu = QPopupMenu(self)
        itm1 = self.menu.insertItem(self.trUtf8('Generate translations'), self.handleGeneration)
        itm2 = self.menu.insertItem(self.trUtf8('Generate translations (with obsolete)'), self.handleGenerationObsolete)
        itm3 = self.menu.insertItem(self.trUtf8('Open in Qt-Linguist'), self.handleOpen)
        itm4 = self.menu.insertItem(self.trUtf8('Release translations'), self.handleRelease)
        self.menu.insertSeparator()
        itm = self.menu.insertItem(self.trUtf8('Remove from project'), self.handleRemove)
        self.menuItems.append(itm)
        itm = self.menu.insertItem(self.trUtf8('Delete'), self.handleDelete)
        self.menuItems.append(itm)
        self.menu.insertSeparator()
        self.menu.insertItem(self.trUtf8('Add translation...'), self.project.addLanguage)
        self.addVCSMenu(self.menu)

        self.backMenu = QPopupMenu(self)
        bitm1 = self.backMenu.insertItem(self.trUtf8('Generate translations'), self.handleGeneration)
        bitm2 = self.backMenu.insertItem(self.trUtf8('Generate translations (with obsolete)'), self.handleGenerationObsolete)
        bitm3 = self.backMenu.insertItem(self.trUtf8('Release translations'), self.handleRelease)
        self.backMenu.insertSeparator()
        self.backMenu.insertItem(self.trUtf8('Add translation...'), self.project.addLanguage)
        self.backMenu.setEnabled(0)

        if self.qtdir == None:
            self.menu.setItemEnabled(itm1, 0)
            self.menu.setItemEnabled(itm2, 0)
            self.menu.setItemEnabled(itm3, 0)
            self.menu.setItemEnabled(itm4, 0)
            self.backMenu.setItemEnabled(bitm1, 0)
            self.backMenu.setItemEnabled(bitm2, 0)
            self.backMenu.setItemEnabled(bitm3, 0)

        self.connect(self.menu,SIGNAL('aboutToShow()'),self.handlePopupMenu)
        
    def handlePopupMenu(self):
        """
        Private slot called by the pyMenu aboutToShow signal.
        """
        self.handleShowPopupMenu(self.menu)
        
    def handleRemove(self):
        """
        Private method to remove a translation from the project.
        """
        itm = self.currentItem()
        fn = str(itm.text(0))
        self.takeItem(itm)
        self.children.remove(itm)
        del itm
        
        self.project.removeLanguage(fn)
        
    def handleDelete(self):
        """
        Private method to delete a translation file from the project.
        """
        itm = self.currentItem()
        fn = str(itm.text(0))
        res = QMessageBox.warning(self.parent(),
            self.trUtf8("Delete translation"),
            self.trUtf8("Dou you really want to delete the "
                "translation for <b>%1</b> from the project?")
                .arg(fn),
            self.trUtf8("&Yes"), self.trUtf8("&No"), None, 1)
        if res == 0:
            if self.project.deleteLanguage(fn):
                self.takeItem(itm)
                self.children.remove(itm)
                del itm
            
    def writeTempProjectFile(self):
        """
        Private method to write a temporary project file suitable for pylupdate and lrelease.
        
        Returns
        
            flag indicating success
        """
        path, ext = os.path.splitext(self.project.pfile)
        pfile = '%s.e3t' % path
        
        try:
            pf = open(pfile, "wb")
            for key in ["SOURCES", "TRANSLATIONS"]:
                list = self.project.pdata[key]
                if len(list) > 0:
                    pf.write('%s = ' % key)
                    last = len(list) - 1
                    if last > 0:
                        pf.write('%s \\%s' % (list[0], os.linesep))
                        for i in range(1, last):
                            pf.write('\t%s \\%s' % (list[i], os.linesep))
                        pf.write('\t%s %s%s' % (list[last], os.linesep, os.linesep))
                    else:
                        pf.write('%s %s%s' % (list[0], os.linesep, os.linesep))
                            
            pf.close()
            self.tmpProject = pfile
            return 1
        except:
            QMessageBox.critical(None,
                self.trUtf8("Write temporary project file"),
                self.trUtf8("The temporary project file <b>%1</b> could not be written.")
                    .arg(pfile),
                QMessageBox.Abort, QMessageBox.NoButton, QMessageBox.NoButton)
            self.tmpProject = None
            return 0
            
    def handleStdout(self):
        """
        Private slot to handle the readyReadStdout signal of the pylupdate/lrelease process.
        """
        if self.pylupdateProc is not None:
            while self.pylupdateProc.canReadLineStdout():
                s = QString('pylupdate: ')
                s.append(self.pylupdateProc.readLineStdout())
                self.emit(PYSIGNAL('appendStdout'), (s,))
        elif self.lreleaseProc is not None:
            while self.lreleaseProc.canReadLineStdout():
                s = QString('lrelease: ')
                s.append(self.lreleaseProc.readLineStdout())
                self.emit(PYSIGNAL('appendStdout'), (s,))
        
    def handleStderr(self):
        """
        Private slot to handle the readyReadStderr signal of the pylupdate/lrelease process.
        """
        if self.pylupdateProc is not None:
            while self.pylupdateProc.canReadLineStderr():
                s = QString('pylupdate: ')
                s.append(self.pylupdateProc.readLineStderr())
                self.emit(PYSIGNAL('appendStderr'), (s,))
        elif self.lreleaseProc is not None:
            while self.lreleaseProc.canReadLineStderr():
                s = QString('lrelease: ')
                s.append(self.lreleaseProc.readLineStderr())
                self.emit(PYSIGNAL('appendStderr'), (s,))
        
    def handleGenerateTSFileDone(self):
        """
        Private slot to handle the processExit signal of the pylupdate/lrelease process.
        """
        self.pylupdateProcRunning = 0
        if self.pylupdateProc.normalExit():
            QMessageBox.information(None,
                self.trUtf8("Translation file generation"),
                self.trUtf8("The generation of the translation files (*.ts) was successful."),
                QMessageBox.Ok)
        else:
            QMessageBox.critical(None,
                self.trUtf8("Translation file generation"),
                self.trUtf8("The generation of the translation files (*.ts) has failed."),
                QMessageBox.Ok, QMessageBox.NoButton, QMessageBox.NoButton)
        self.pylupdateProc = None
        try:
            os.remove(self.tmpProject)
        except:
            pass
        self.tmpProject = None
                
    def generateTSFile(self, noobsolete = 0):
        """
        Private method used to run pyludate to generate the .ts files.
        
        Arguments
        
            noobsolete -- flag indicating whether obsolete entries should be kept
        """
        # generate a minimal temporary projectfile suitable for pylupdate
        ok = self.writeTempProjectFile()
        if not ok:
            return
            
        self.pylupdateProc = QProcess(self)
        
        pylupdate = 'pylupdate'
        if sys.platform == "win32":
            pylupdate = pylupdate + '.exe'
        self.pylupdateProc.addArgument(pylupdate)
        
        if noobsolete:
            self.pylupdateProc.addArgument('-noobsolete')
        
        self.pylupdateProc.addArgument('-verbose')
        self.pylupdateProc.addArgument(self.tmpProject)
        self.pylupdateProc.setWorkingDirectory(QDir(self.project.ppath))
        self.connect(self.pylupdateProc, SIGNAL('processExited()'), self.handleGenerateTSFileDone)
        self.connect(self.pylupdateProc, SIGNAL('readyReadStdout()'), self.handleStdout)
        self.connect(self.pylupdateProc, SIGNAL('readyReadStderr()'), self.handleStderr)
        
        self.pylupdateProcRunning = 1
        if not self.pylupdateProc.start():
            QMessageBox.critical(self,
                self.trUtf8('Process Generation Error'),
                self.trUtf8(
                    'Could not start pylupdate.<br>'
                    'Ensure that it is in the search path.'
                ),
                self.trUtf8('OK'))
        
    def handleGeneration(self):
        """
        Private method to generate the translation files (.ts) for Qt Linguist.
        
        All obsolete strings are removed from the .ts file.
        """
        self.generateTSFile(1)
        
    def handleGenerationObsolete(self):
        """
        Private method to generate the translation files (.ts) for Qt Linguist.
        
        Obsolete strings are kept.
        """
        self.generateTSFile(0)
        
    def handleReleaseDone(self):
        """
        Private slot to handle the processExit signal of the pylupdate/lrelease process.
        """
        self.lreleaseProcRunning = 0
        if self.lreleaseProc.normalExit():
            QMessageBox.information(None,
                self.trUtf8("Translation file release"),
                self.trUtf8("The release of the translation files (*.qm) was successful."),
                QMessageBox.Ok)
        else:
            QMessageBox.critical(None,
                self.trUtf8("Translation file release"),
                self.trUtf8("The release of the translation files (*.qm) has failed."),
                QMessageBox.Ok, QMessageBox.NoButton, QMessageBox.NoButton)
        self.lreleaseProc = None
        try:
            os.remove(self.tmpProject)
        except:
            pass
        self.tmpProject = None
        
    def handleRelease(self):
        """
        Private method to release the translation files (.qm).
        """
        # generate a minimal temporary projectfile suitable for lrelease
        ok = self.writeTempProjectFile()
        if not ok:
            return
            
        self.lreleaseProc = QProcess(self)
        
        lrelease = os.path.join(self.qtdir, 'bin', 'lrelease')
        if sys.platform == "win32":
            lrelease = lrelease + '.exe'
        self.lreleaseProc.addArgument(lrelease)
        
        self.lreleaseProc.addArgument('-verbose')
        self.lreleaseProc.addArgument(self.tmpProject)
        self.lreleaseProc.setWorkingDirectory(QDir(self.project.ppath))
        self.connect(self.lreleaseProc, SIGNAL('processExited()'), self.handleReleaseDone)
        self.connect(self.lreleaseProc, SIGNAL('readyReadStdout()'), self.handleStdout)
        self.connect(self.lreleaseProc, SIGNAL('readyReadStderr()'), self.handleStderr)
        
        self.lreleaseProcRunning = 1
        if not self.lreleaseProc.start():
            QMessageBox.critical(self,
                self.trUtf8('Process Generation Error'),
                self.trUtf8(
                    'Could not start lrelease.<br>'
                    'Ensure that it is available as <b>%1</b>.'
                ).arg(lrelease),
                self.trUtf8('OK'))
        
    def handleProjectLanguageAdded(self, fn):
        """
        Private slot to handle the projectLanguageAdded signal.
        """
        fname = os.path.join(self.project.ppath, fn)
        dt, ext = os.path.splitext(fn)
        dt = dt.split('_', 1)[-1]
        node = BrowserFile(self, fname, None, 1, dt)
        self.children.append(node)
        self.nodeAdded(node, fname)

        
    def handleVCSUpdate(self):
        """
        Private slot called by the context menu to add a file to the VCS repository.
        """
        PBrowser.handleVCSUpdate(self)
        itm = self.currentItem()
        fn = str(itm.fileName())
        qfn, ext = os.path.splitext(fn)
        qfn = '%s.qm' % qfn
        self.project.vcs.vcsUpdate(qfn)
        
    def handleVCSCommit(self):
        """
        Private slot called by the context menu to commit the changes to the VCS repository.
        """
        PBrowser.handleVCSCommit(self)
        itm = self.currentItem()
        fn = str(itm.fileName())
        qfn, ext = os.path.splitext(fn)
        qfn = '%s.qm' % qfn
        self.project.vcs.vcsCommit(qfn, '')
        
    def handleVCSAdd(self):
        """
        Private slot called by the context menu to add the selected language to the VCS repository.
        """
        itm = self.currentItem()
        fn = str(itm.fileName())
        qfn, ext = os.path.splitext(fn)
        qfn = '%s.qm' % qfn
        
        # create empty files if they don't exist
        if not os.path.exists(fn):
            try:
                f = open(fn, 'w')
                f.close()
            except:
                pass
        if not os.path.exists(qfn):
            try:
                f = open(qfn, 'w')
                f.close()
            except:
                pass
                
        self.project.vcs.vcsAdd(fn)
        self.project.vcs.vcsAddBinary(qfn)
        self.updateVCSStatus(itm, fn)
        
    def handleVCSRemove(self):
        """
        Private slot called by the context menu to remove the selected language from the VCS repository.
        """
        itm = self.currentItem()
        fn = str(itm.fileName())
        qfn, ext = os.path.splitext(fn)
        qfn = '%s.qm' % qfn
        self.project.vcs.vcsRemove(qfn)
        PBrowser.handleVCSRemove(self)
        
        
class ProjectOthersBrowser(PBrowser):
    """
    A class used to display the parts of the project, that don't fit the other categories.
    
    Via the context menu that is displayed by a right 
    click the user can select various actions on the selected file or 
    directory.
    
    Signals
    
        pythonFile(string) -- emitted to open a file
        
        closeSourceWindow(string) -- emitted after a file has been removed/deleted
            from the project
    """
    def __init__(self,project,parent=None):
        """
        Constructor
        
        Arguments
        
            project -- reference to the project object
            
            parent -- parent widget of this browser (QWidget)
        """
        PBrowser.__init__(self,project,"OTHERS",parent)
    
        self.setCaption(self.trUtf8('Others'))

        QWhatsThis.add(self,self.trUtf8(
            """<b>Project Others Browser</b>"""
            """<p>This allows to easily see all other files and directories"""
            """ contained in the current project. Several actions can be"""
            """ executed via the context menu. The entry which is registered"""
            """ in the project is shown in a different colour.</p>"""
        ))
        
    def createPopupMenus(self):
        """
        Private overloaded method to generate the popup menu.
        """
        PBrowser.createPopupMenus(self)
        
        self.menu.insertSeparator()
        itm = self.menu.insertItem(self.trUtf8('Remove from project'), self.handleRemove)
        self.menuItems.append(itm)
        itm = self.menu.insertItem(self.trUtf8('Delete'), self.handleDelete)
        self.menuItems.append(itm)
        self.menu.insertSeparator()
        self.menu.insertItem(self.trUtf8('Add file...'), self.addFile)
        self.menu.insertItem(self.trUtf8('Add directory...'), self.addDir)
        self.addVCSMenu(self.menu)

        self.backMenu = QPopupMenu(self)
        self.backMenu.insertItem(self.trUtf8('Add file...'), self.addFile)
        self.backMenu.insertItem(self.trUtf8('Add directory...'), self.addDir)
        self.backMenu.setEnabled(0)

        self.connect(self.menu,SIGNAL('aboutToShow()'),self.handlePopupMenu)
        
    def addVCSMenu(self, menu):
        """
        Public method used to add the VCS menu to all project browsers.
        
        Arguments
        
            menu -- reference to the menu to ammend (QPopupMenu)
        """
        menu.insertSeparator()
        self.vcsMenuItems = []
        itm = menu.insertItem(self.trUtf8('Update from repository'), self.handleVCSUpdate)
        self.vcsMenuItems.append(itm)
        itm = menu.insertItem(self.trUtf8('Commit changes to repository...'), self.handleVCSCommit)
        self.vcsMenuItems.append(itm)
        self.vcsMenuAdd = menu.insertItem(self.trUtf8('Add to repository'), self.handleVCSAdd)
        self.vcsMenuAddBinary = menu.insertItem(self.trUtf8('Add to repository (binary)'), self.handleVCSAddBinary)
        self.vcsMenuAddTree = menu.insertItem(self.trUtf8('Add tree to repository'), self.handleVCSAddTree)
        itm = menu.insertItem(self.trUtf8('Remove from repository (and disk)'), self.handleVCSRemove)
        self.vcsMenuItems.append(itm)
        itm = menu.insertItem(self.trUtf8('Show log'), self.handleVCSLog)
        self.vcsMenuItems.append(itm)
        itm = menu.insertItem(self.trUtf8('Show difference to repository'), self.handleVCSDiff)
        self.vcsMenuItems.append(itm)
        itm = menu.insertItem(self.trUtf8('Show history'), self.handleVCSHistory)
        self.vcsMenuItems.append(itm)
        itm = menu.insertItem(self.trUtf8('Show status'), self.handleVCSStatus)
        self.vcsMenuItems.append(itm)
        
    def handleContextMenu(self,itm,coord,col):
        """
        Private slot to show the context menu of the listview.
        
        Arguments
        
            itm -- the selected item (QListViewItem)
            
            coord -- the position of the mouse cursor (QPoint)
            
            col -- the column of the mouse cursor (int)
        """
        try:
            if isinstance(itm,BrowserFile) or isinstance(itm,ProjectBrowserDirectory):
                self.menu.popup(coord)
            else:
                self.backMenu.popup(coord)
        except:
            pass
            
    def handlePopupMenu(self):
        """
        Private slot called by the menu aboutToShow signal.
        """
        self.handleShowPopupMenu(self.menu)
        
    def handleShowPopupMenu(self, menu):
        """
        Slot called before the context menu is shown. 
        
        It enables/disables
        the VCS menu entries depending on the overall VCS status and the
        file status.
        """
        if self.project.vcs is None:
            for itm in self.vcsMenuItems:
                menu.setItemEnabled(itm, 0)
            menu.setItemEnabled(self.vcsMenuAdd, 0)
            menu.setItemEnabled(self.vcsMenuAddBinary, 0)
            menu.setItemEnabled(self.vcsMenuAddTree, 0)
            for itm in self.menuItems:
                menu.setItemEnabled(itm, 1)
        else:
            if str(self.currentItem().text(1)) == self.project.vcs.vcsName():
                for itm in self.vcsMenuItems:
                    menu.setItemEnabled(itm, 1)
                menu.setItemEnabled(self.vcsMenuAdd, 0)
                menu.setItemEnabled(self.vcsMenuAddBinary, 0)
                menu.setItemEnabled(self.vcsMenuAddTree, 0)
                for itm in self.menuItems:
                    menu.setItemEnabled(itm, 0)
            else:
                for itm in self.vcsMenuItems:
                    menu.setItemEnabled(itm, 0)
                menu.setItemEnabled(self.vcsMenuAdd, 1)
                menu.setItemEnabled(self.vcsMenuAddBinary, 1)
                if os.path.isdir(str(self.currentItem().fileName())):
                    menu.setItemEnabled(self.vcsMenuAddTree, 1)
                else:
                    menu.setItemEnabled(self.vcsMenuAddTree, 0)
                for itm in self.menuItems:
                    menu.setItemEnabled(itm, 1)
        
    def handleOpen(self):
        """
        Private slot to handle the open popup menu entry.
        """
        itm = self.currentItem()
        
        try:
            if isinstance(itm, BrowserFile):
                self.emit(PYSIGNAL('pythonFile'),(itm.fileName(),))
        except:
            pass
            
    def handleRemove(self):
        """
        Private slot to remove the selected entry from the OTHERS project data area.
        """
        itm = self.currentItem()
        fn2 = str(itm.fileName())
        fn = fn2.replace(self.project.ppath+os.sep, '')
        if fn in self.project.pdata["OTHERS"]:
            if isinstance(itm, BrowserFile):
                self.emit(PYSIGNAL('closeSourceWindow'), (fn2,))
                
            self.removeNode(itm)
            self.children.remove(itm)
            del itm
            
            self.project.pdata["OTHERS"].remove(fn)
            self.project.setDirty(1)
        else:
            QMessageBox.information(None,
                self.trUtf8("Remove file/directory"),
                self.trUtf8("The selected entry is not recorded in the project."),
                self.trUtf8("OK"))
        
    def handleDelete(self):
        """
        Private method to delete the selected entry from the OTHERS project data area.
        """
        itm = self.currentItem()
        fn2 = str(itm.fileName())
        fn = fn2.replace(self.project.ppath+os.sep, '')
        if fn in self.project.pdata["OTHERS"]:
            res = QMessageBox.warning(self.parent(),
                self.trUtf8("Delete file/directory"),
                self.trUtf8("Dou you really want to delete <b>%1</b> from the project?")
                    .arg(fn),
                self.trUtf8("&Yes"), self.trUtf8("&No"), None, 1)
            if res == 0:
                if isinstance(itm, BrowserFile):
                    self.emit(PYSIGNAL('closeSourceWindow'), (fn2,))
                    try:
                        os.remove(fn2)
                    except:
                        QMessageBox.critical(None,
                            self.trUtf8("Delete file"),
                            self.trUtf8("The selected file <b>%1</b> could not be deleted.")
                                .arg(fn),
                            QMessageBox.Ok, QMessageBox.NoButton, 
                            QMessageBox.NoButton)
                        return
                elif isinstance(itm, ProjectBrowserDirectory):
                    try:
                        shutil.rmtree(fn2)
                    except:
                        QMessageBox.critical(None,
                            self.trUtf8("Delete directory"),
                            self.trUtf8("The selected directory <b>%1</b> could not be deleted.")
                                .arg(fn),
                            QMessageBox.Ok, QMessageBox.NoButton, 
                            QMessageBox.NoButton)
                        return
                
                self.removeNode(itm)
                self.children.remove(itm)
                del itm
                
                self.project.pdata["OTHERS"].remove(fn)
                self.project.setDirty(1)
        else:
            QMessageBox.information(None,
                self.trUtf8("Delete file/directory"),
                self.trUtf8("The selected entry is not recorded in the project."),
                self.trUtf8("OK"))

    def addNode(self, name):
        """
        Public slot to add a node to this browser.
        
        Arguments
        
            name -- filename or directory of this node
        """
        name = str(name)
        if not os.path.isabs(name):
            fname = os.path.join(self.project.ppath, name)
        else:
            fname = name
        parent, dt = self.findParentNode(name)
        if os.path.isdir(fname):
            node = ProjectBrowserDirectory(parent, fname, None, 0, 1, self.project.vcs)
        else:
            node = BrowserFile(parent, fname, None, 1, dt, 1)
        self.children.append(node)
        self.nodeAdded(node, fname)
        
    def addToOthers(self, fn):
        """
        Private method to add file/directory to the OTHERS project data.
        
        Arguments
        
            fn -- filename or directoryname to add
        """
        if not fn.isEmpty():
            fn = str(fn)
            
            # if it is below the project directory, make it relative to that
            fn = fn.replace(self.project.ppath+os.sep, '')
            
            # if it ends with the directory separator character, remove it
            if fn.endswith(os.sep):
                fn = fn[:-1]
            self.project.pdata['OTHERS'].append(fn)
            self.project.setDirty(1)
            self.addNode(fn)
        
    def addFile(self):
        """
        Private slot to add a file to the OTHERS project data.
        """
        fn = QFileDialog.getOpenFileName(self.project.ppath,
                self.trUtf8("All files (*)"),
                self, None, self.trUtf8("Add file"), None, 0)
        self.addToOthers(fn)
        
    def addDir(self):
        """
        Private slot to add a directory to the OTHERS project data.
        """
        dn = QFileDialog.getExistingDirectory(self.project.ppath,
                self, None, self.trUtf8("Add directory"), 1, 0)
        self.addToOthers(dn)
        
    def handleVCSAdd(self):
        """
        Private slot called by the context menu.
        
        It is used to add the selected
        file/directory to the VCS repository.
        """
        itm = self.currentItem()
        fn = str(itm.fileName())
        self.project.vcs.vcsAdd(fn, os.path.isdir(fn))
        self.updateVCSStatus(itm, fn)
        
    def handleVCSAddBinary(self):
        """
        Private slot called by the context menu.
        
        It is used to add the selected
        file/directory in binary mode to the VCS repository.
        """
        itm = self.currentItem()
        fn = str(itm.fileName())
        self.project.vcs.vcsAddBinary(fn, os.path.isdir(fn))
        self.updateVCSStatus(itm, fn)
        
    def handleVCSAddTree(self):
        """
        Private slot called by the context menu.
        
        It is used to add the selected
        directory tree to the VCS repository.
        """
        itm = self.currentItem()
        fn = str(itm.fileName())
        self.project.vcs.vcsAddTree(fn)
        self.updateVCSStatus(itm, fn)
        itm.setOpen(0)
        
    def handleVCSRemove(self):
        """
        Private slot called by the context menu.
        
        It is used to remove the selected
        file/directory from the VCS repository.
        """
        itm = self.currentItem()
        fn = str(itm.fileName())
        res = QMessageBox.warning(None,
            self.trUtf8("Remove file/directory from repository"),
            self.trUtf8("Dou you really want to remove <b>%1</b> from the repository (and disk)?")
                .arg(fn),
            self.trUtf8("&Yes"), self.trUtf8("&No"), None, 1)
        if res == 0:
            self.project.vcs.vcsRemove(fn)
            if not os.path.exists(fn) or os.path.isdir(fn):
                self.handleRemove() # remove file from project

