# -*- coding: utf-8 -*-

# Copyright (c) 2003 Detlev Offenbach <detlev@die-offenbachs.de>
#

"""
Package implementing various functions/classes needed everywhere within eric3. 
"""

import os
import re
import string

coding_re = re.compile(r"coding[:=]\s*([-\w_.]+)")
UTF8_BOM = '\xef\xbb\xbf'
def get_coding(text):
    """
    Function to get the coding of a text.
    
    Arguments
    
        text -- text to inspect
        
    Returns
    
        coding string
    """
    for l in text.splitlines()[:2]:
        m = coding_re.search(l)
        if m:
            return m.group(1)
    return None

def decode(text):
    """
    Function to decode a text.
    
    Arguments
    
        text -- text to decode
        
    Returns
    
        decoded text
    """
    try:
        if text.startswith(UTF8_BOM):
            # UTF-8 with BOM
            return unicode(text[3:], 'utf-8'), 'utf-8-bom'
        coding = get_coding(text)
        if coding:
            return unicode(text, coding), coding
    except (UnicodeError, LookupError):
        pass
    # Assume Latin-1
    return unicode(text, "latin-1"), 'latin-1-guessed'

def encode(text, orig_coding):
    """
    Function to encode a text.
    
    Arguments
    
        text -- text to encode
        
        orig_coding -- type of the original coding
        
    Returns
    
        encoded text
    """
    if orig_coding == 'utf-8-bom':
        return UTF8_BOM + text.encode("utf-8")
    # Try saving as ASCII
    try:
        return text.encode('ascii')
    except UnicodeError:
        pass
    # Try declared coding spec
    coding = get_coding(text)
    try:
        if coding:
            return text.encode(coding)
    except (UnicodeError, LookupError):
        # Error: Declared encoding is incorrect
        pass
    try:
        if orig_coding == 'latin-1-guessed':
            return text.encode('latin-1')
    except UnicodeError:
        pass
    # Save as UTF-8 without BOM
    return text.encode('utf-8')
    
def normcasepath(path):
    """
    Function returning a path, that is normalized with respect to its case and references.
    """
    return os.path.normcase(os.path.normpath(path))
    
def normabspath(path):
    """
    Function returning a normalized, absolute path.
    """
    return os.path.abspath(path)
    
def normjoinpath(a, *p):
    """
    Function returning a normalized path of the joined parts passed into it.
    """
    return os.path.normpath(os.path.join(a, *p))
    
def normabsjoinpath(a, *p):
    """
    Function returning a normalized, absolute path of the joined parts passed into it.
    """
    return os.path.abspath(os.path.join(a, *p))
    
def isinpath(file):
    """
    Function to check for a file.
    
    Returns
    
        flag to indicate, if the executable file is accessible
        via the searchpath defined by the PATH environment variable.
    """
    if os.path.isabs(file):
        return os.access(file, os.X_OK)
        
    path = os.getenv('PATH')
    
    # environment variable not defined
    if path is None:
        return 0
        
    dirs = string.split(path, os.pathsep)
    for dir in dirs:
        if os.access(os.path.join(dir, file), os.X_OK):
            return 1
            
    return 0

def samepath(f1, f2):
    """
    Function to compare two paths.
    
    Returns
    
        flag indicating whether the two paths represent the
        same path on disk.
    """
    if f1 is None or f2 is None:
        return 0
        
    if normcasepath(f1) == normcasepath(f2):
        return 1
        
    return 0
        
def direntries(path, filesonly=0):
    """
    Function returning a list of all files and directories.
    
    Arguments
    
        path -- root of the tree to check
        
        filesonly -- flag indicating that only files are wanted

    Returns
    
        list of all files and directories in the tree rooted
        at path. The names are expanded to start with path. 
    """
    if filesonly:
        files = []
    else:
        files = [path]
    entries = os.listdir(path)
    for entry in entries:
        if entry == 'CVS':
            continue
        entry = os.path.join(path, entry)
        if os.path.isdir(entry):
            files += direntries(entry, filesonly)
        else:
            files.append(entry)
    return files
