# -*- coding: utf-8 -*-

# Copyright (c) 2002, 2003 Detlev Offenbach <detlev@die-offenbachs.de>
#

"""
Module implementing the listspace viewmanager class.
"""

import os

from qt import *

from ViewManager import ViewManager
from QScintilla.EditorIcons import IconFileModified

class Listspace(QSplitter, ViewManager):
    """
    Class implementing the listspace viewmanager class.
    
    Signals
    
        lastEditorClosed -- emitted after the last editor window was closed
        
        editorOpened -- emitted after an editor window was opened
        
        editorSaved -- emitted after an editor window was saved
    """
    def __init__(self,parent, ui, dbs):
        """
        Constructor
        
        Arguments
        
            parent -- parent widget (QWidget)
            
            ui -- reference to the main user interface
            
            dbs -- reference to the debug server object
        """
        QSplitter.__init__(self,parent)
        ViewManager.__init__(self, ui, dbs)
        self.viewlist = QListBox(self)
        self.setResizeMode(self.viewlist, QSplitter.KeepSize)
        self.stack = QWidgetStack(self)
        self.connect(self.viewlist, SIGNAL("highlighted(int)"),
                     self.__showViewByIndex)
                     
        dummy = self.trUtf8('dummy')
        
    def canCascade(self):
        """
        Public method to signal if cascading of managed windows is available.
        
        Returns
        
            flag indicating cascading of windows is available
        """
        return 0
        
    def canTile(self):
        """
        Public method to signal if tiling of managed windows is available.
        
        Returns
        
            flag indicating tiling of windows is available
        """
        return 0
    
    def tile(self):
        """
        Public method to tile the managed windows.
        """
        pass
        
    def cascade(self):
        """
        Public method to cascade the managed windows.
        """
        pass
        
    def removeAllViews(self):
        """
        Private method to remove all views (i.e. windows)
        """
        for i in range(self.viewlist.count() - 1, -1, -1):
            self.viewlist.removeItem(i)
        for win in self.editors:
            self.stack.removeWidget(win)
            win.closeIt()
            
    def removeView(self, win):
        """
        Private method to remove a view (i.e. window)
        
        Arguments
        
            win -- editor window to be removed
        """
        ind = self.editors.index(win)
        self.viewlist.removeItem(ind)
        self.stack.removeWidget(win)
        win.closeIt()
        if ind > 0:
            ind -= 1
            lind = ind
        else:
            if len(self.editors) > 1:
                ind = 1
                lind = 0
            else:
                return
        self.stack.raiseWidget(self.editors[ind])
        self.viewlist.setSelected(lind, 1)
    
    def addView(self, win, fn=None):
        """
        Private method to add a view (i.e. window)
        
        Arguments
        
            win -- editor window to be added
            
            fn -- filename of this editor
        """
        if fn is None:
            self.untitledCount += 1
            self.viewlist.insertItem(
                self.trUtf8("Untitled %1").arg(self.untitledCount), 
                len(self.editors) - 1)
        else:
            txt = os.path.basename(fn)
            if not QFileInfo(fn).isWritable():
                txt = '%s (ro)' % txt
            self.viewlist.insertItem(txt, len(self.editors) - 1)
        self.stack.addWidget(win, len(self.editors) - 1)
        self.stack.raiseWidget(win)
        self.viewlist.setSelected(len(self.editors) - 1, 1)
        win.setFocus()
    
    def showView(self, win, fn=None):
        """
        Private method to show a view (i.e. window)
        
        Arguments
        
            win -- editor window to be shown
            
            fn -- filename of this editor
        """
        self.stack.raiseWidget(win)
        self.viewlist.setSelected(self.editors.index(win), 1)
        win.setFocus()
        
    def __showViewByIndex(self, index):
        """
        Internal slot called to show a view (i.e. window) selected in the list.
        
        Arguments
        
            index -- index of the selected entry (int)
        """
        self.showView(self.editors[index])
        self.checkActions(self.editors[index])

    def activeWindow(self):
        """
        Private method to return the active (i.e. current) window.
        
        Returns
        
            reference to the active editor
        """
        return self.stack.visibleWidget()
        
    def handleShowWindowMenu(self, windowMenu):
        """
        Private method to set up the viewmanager part of the Window menu.
        
        Arguments
        
            windowMenu -- reference to the window menu
        """
        pass
        
    def initWindowActions(self):
        """
        Define the user interface actions for window handling.
        """
        pass

    def setEditorName(self, editor, newName):
        """
        Change the displayed name of the editor.
        
        Arguments
        
            editor -- editor window to be changed
            
            newName -- new name to be shown (string or QString)
        """
        idx = self.editors.index(editor)
        self.viewlist.changeItem(os.path.basename(newName), idx)

    def handleModificationStatusChanged(self, m, editor):
        """
        Private slot to handle the modificationStatusChanged signal.
        
        Arguments
        
            m -- flag indicating the modification status (boolean)
            
            editor -- editor window changed
        """
        idx = self.editors.index(editor)
        if m:
            self.viewlist.changeItem(IconFileModified, self.viewlist.text(idx), idx)
        else:
            self.viewlist.changeItem(self.viewlist.text(idx), idx)
        self.checkActions(editor)
        
