# -*- coding: utf-8 -*-

# Copyright (c) 2002, 2003 Detlev Offenbach <detlev@die-offenbachs.de>
#

"""
Module implementing a tabbed viewmanager class.
"""

import os

from qt import *

from ViewManager import ViewManager
from QScintilla.EditorIcons import IconFileModified
from Icons.Icons import IconEmpty

class Tabview(QTabWidget, ViewManager):
    """
    Class implementing a tabbed viewmanager class.
    
    Signals
    
        lastEditorClosed -- emitted after the last editor window was closed
        
        editorOpened -- emitted after an editor window was opened
        
        editorSaved -- emitted after an editor window was saved
    """
    def __init__(self,parent, ui, dbs):
        """
        Constructor
        
        Arguments
        
            parent -- parent widget (QWidget)
            
            ui -- reference to the main user interface
            
            dbs -- reference to the debug server object
        """
        QTabWidget.__init__(self,parent)
        ViewManager.__init__(self, ui, dbs)
        self.connect(self, SIGNAL('currentChanged(QWidget*)'),
            self.handleCurrentChanged)
            
        dummy = self.trUtf8('dummy')
        
    def canCascade(self):
        """
        Public method to signal if cascading of managed windows is available.
        
        Returns
        
            flag indicating cascading of windows is available
        """
        return 0
        
    def canTile(self):
        """
        Public method to signal if tiling of managed windows is available.
        
        Returns
        
            flag indicating tiling of windows is available
        """
        return 0
        
    def tile(self):
        """
        Public method to tile the managed windows.
        """
        pass
        
    def cascade(self):
        """
        Public method to cascade the managed windows.
        """
        pass
        
    def removeAllViews(self):
        """
        Private method to remove all views (i.e. windows)
        """
        for win in self.editors:
            self.removeView(win)
            
    def removeView(self, win):
        """
        Private method to remove a view (i.e. window)
        
        Arguments
        
            win -- editor window to be removed
        """
        self.removePage(win)
        win.closeIt()
    
    def addView(self, win, fn=None):
        """
        Private method to add a view (i.e. window)
        
        Arguments
        
            win -- editor window to be added
            
            fn -- filename of this editor
        """
        win.show()
        if fn is None:
            self.untitledCount += 1
            self.addTab(win, self.trUtf8("Untitled %1").arg(self.untitledCount))
        else:
            txt = os.path.basename(fn)
            if not QFileInfo(fn).isWritable():
                txt = '%s (ro)' % txt
            self.addTab(win, txt)
            self.setTabToolTip(win, os.path.dirname(fn))
        self.showPage(win)
        win.setFocus()
    
    def showView(self, win, fn=None):
        """
        Private method to show a view (i.e. window)
        
        Arguments
        
            win -- editor window to be shown
            
            fn -- filename of this editor
        """
        win.show()
        self.showPage(win)
        win.setFocus()
    
    def activeWindow(self):
        """
        Private method to return the active (i.e. current) window.
        
        Returns
        
            reference to the active editor
        """
        return self.currentPage()
        
    def handleShowWindowMenu(self, windowMenu):
        """
        Private method to set up the viewmanager part of the Window menu.
        
        Arguments
        
            windowMenu -- reference to the window menu
        """
        pass
        
    def initWindowActions(self):
        """
        Define the user interface actions for window handling.
        """
        pass
        
    def setEditorName(self, editor, newName):
        """
        Change the displayed name of the editor.
        
        Arguments
        
            editor -- editor window to be changed
            
            newName -- new name to be shown (string or QString)
        """
        self.changeTab(editor, os.path.basename(str(newName)))
        self.setTabToolTip(editor, os.path.dirname(str(newName)))

    def handleModificationStatusChanged(self, m, editor):
        """
        Private slot to handle the modificationStatusChanged signal.
        
        Arguments
        
            m -- flag indicating the modification status (boolean)
            
            editor -- editor window changed
        """
        if m:
            self.setTabIconSet(editor, QIconSet(IconFileModified))
        else:
            self.setTabIconSet(editor, QIconSet(IconEmpty))
        self.checkActions(editor)
        
    def handleCurrentChanged(self, editor):
        """
        Private slot to handle the currentChanged signal.
        
        Arguments
        
            editor -- selected editor window
        """
        self.checkActions(editor)
        editor.setFocus()
