# -*- coding: utf-8 -*-

# Copyright (c) 2002, 2003 Detlev Offenbach <detlev@die-offenbachs.de>
#

"""
Module implementing the viewmanager base class.
"""

import os

from qt import *

import Preferences
from QScintilla.Editor import Editor
from Icons.Icons import *
from Project.ProjectIcons import IconProjectFind
from QScintilla.EditorIcons import *
from QScintilla.GotoDialog import GotoDialog
from QScintilla.SearchReplaceDialog import SearchReplaceDialog
from QScintilla.ZoomDialog import ZoomDialog
import Utilities

class ViewManager:
    """
    Base class inherited by all specific viewmanager classes.
    
    It defines the interface to be implemented by specific
    viewmanager classes and all common methods.
    
    Signals
    
        lastEditorClosed -- emitted after the last editor window was closed
        
        editorOpened -- emitted after an editor window was opened
        
        editorSaved -- emitted after an editor window was saved
    """
    def __init__(self, ui, dbs):
        """
        Constructor
        
        Arguments
        
            ui -- reference to the main user interface
            
            dbs -- reference to the debug server object
        """
        # initialize the instance variables
        self.ui = ui
        self.editors = []
        self.currentEditor = None
        self.dbs = dbs
        self.untitledCount = 0
        self.srHistory = {"search" : QStringList(), "replace" : QStringList()}
        
        self.recent = QStringList()
        rp, ok = Preferences.Prefs.settings.readListEntry('/eric3/Recent/Sources')
        if ok:
            self.recent = rp
            
        self.searchDlg = SearchReplaceDialog(0, self, ui)
        self.replaceDlg = SearchReplaceDialog(1, self, ui)
        
    def setSbInfo(self, sbFile, sbLine, sbPos, sbWritable):
        """
        Public method to transfer statusbar info from the user interface to viewmanager.
        
        Arguments
        
            sbFile -- reference to the file part of the statusbar
            
            sbLine -- reference to the line number part of the statusbar
            
            sbPos -- reference to the character position part of the statusbar
            
            sbWritable -- reference to the writability indicator part of the statusbar
        """
        self.sbFile = sbFile
        self.sbLine = sbLine
        self.sbPos = sbPos
        self.sbWritable = sbWritable
        self.setSbFile()
        
    def canCascade(self):
        """
        Public method to signal if cascading of managed windows is available.
        
        Returns
        
            flag indicating cascading of windows is available
        """
        raise RuntimeError, 'Not implemented'
        
    def canTile(self):
        """
        Public method to signal if tiling of managed windows is available.
        
        Returns
        
            flag indicating tiling of windows is available
        """
        raise RuntimeError, 'Not implemented'
        
    def tile(self):
        """
        Public method to tile the managed windows.
        """
        raise RuntimeError, 'Not implemented'
    
    def cascade(self):
        """
        Public method to cascade the managed windows.
        """
        raise RuntimeError, 'Not implemented'

    def removeAllViews(self):
        """
        Private method to remove all views (i.e. windows)
        """
        raise RuntimeError, 'Not implemented'
    
    def removeView(self, win):
        """
        Private method to remove a view (i.e. window)
        
        Arguments
        
            win -- editor window to be removed
        """
        raise RuntimeError, 'Not implemented'
    
    def addView(self, win, fn=None):
        """
        Private method to add a view (i.e. window)
        
        Arguments
        
            win -- editor window to be added
            
            fn -- filename of this editor
        """
        raise RuntimeError, 'Not implemented'
    
    def showView(self, win, fn=None):
        """
        Private method to show a view (i.e. window)
        
        Arguments
        
            win -- editor window to be shown
            
            fn -- filename of this editor
        """
        raise RuntimeError, 'Not implemented'
    
    def activeWindow(self):
        """
        Private method to return the active (i.e. current) window.
        
        Returns
        
            reference to the active editor
        """
        raise RuntimeError, 'Not implemented'
    
    def handleShowWindowMenu(self, windowMenu):
        """
        Private method to set up the viewmanager part of the Window menu.
        
        Arguments
        
            windowMenu -- reference to the window menu
        """
        raise RuntimeError, 'Not implemented'
    
    def initWindowActions(self):
        """
        Define the user interface actions for window handling.
        """
        raise RuntimeError, 'Not implemented'

    def setEditorName(self, editor, newName):
        """
        Change the displayed name of the editor.
        
        Arguments
        
            editor -- editor window to be changed
            
            newName -- new name to be shown (string or QString)
        """
        raise RuntimeError, 'Not implemented'
        
    def handleModificationStatusChanged(self, m, editor):
        """
        Private slot to handle the modificationStatusChanged signal.
        
        Arguments
        
            m -- flag indicating the modification status (boolean)
            
            editor -- editor window changed
        """
        raise RuntimeError, 'Not implemented'
        
    #####################################################################
    ## methods above need to be implemented by a subclass
    #####################################################################
    
    def initActions(self):
        """
        Public method defining the user interface actions.
        """
        self.initWindowActions()
        self.initFileActions()
        self.initEditActions()
        self.initSearchActions()
        self.initViewActions()
        
    ##################################################################
    ## Initialize the file related actions, file menu and toolbar
    ##################################################################
    
    def initFileActions(self):
        """
        Private method defining the user interface actions for file handling.
        """
        self.newAct = QAction(self.trUtf8('New'),
                QIconSet(IconNew),
                self.trUtf8('&New'),
                QKeySequence(self.trUtf8("CTRL+N","File|New")),self)
        self.newAct.setStatusTip(self.trUtf8('Open an empty editor window'))
        self.newAct.setWhatsThis(self.trUtf8(
            """<b>New</b>"""
            """<p>An empty editor window will be created.</p>"""
        ))
        self.connect(self.newAct,SIGNAL('activated()'),self.newEditor)
        
        self.openAct = QAction(self.trUtf8('Open'),
                QIconSet(IconOpen),
                self.trUtf8('&Open...'),
                QKeySequence(self.trUtf8("CTRL+O","File|Open")),self)
        self.openAct.setStatusTip(self.trUtf8('Open a Python file'))
        self.openAct.setWhatsThis(self.trUtf8(
            """<b>Open a Python file</b>"""
            """<p>You will be asked for the name of a Python file to be opened"""
            """ in an editor window.</p>"""
        ))
        self.connect(self.openAct,SIGNAL('activated()'),self.handleOpen)
        
        self.closeActGrp = QActionGroup(self)
        
        self.closeAct = QAction(self.trUtf8('Close'),
                QIconSet(IconClose),
                self.trUtf8('&Close'),
                QKeySequence(self.trUtf8("CTRL+W","File|Close")),
                self.closeActGrp)
        self.closeAct.setStatusTip(self.trUtf8('Close the current window'))
        self.closeAct.setWhatsThis(self.trUtf8(
            """<b>Close Window</b>"""
            """<p>Close the current window.</p>"""
        ))
        self.connect(self.closeAct,SIGNAL('activated()'),self.handleClose)

        self.closeAllAct = QAction(self.trUtf8('Close All'),
                self.trUtf8('Clos&e All'),
                0,self.closeActGrp)
        self.closeAllAct.setStatusTip(self.trUtf8('Close all editor windows'))
        self.closeAllAct.setWhatsThis(self.trUtf8(
            """<b>Close All Windows</b>"""
            """<p>Close all editor windows.</p>"""
        ))
        self.connect(self.closeAllAct,SIGNAL('activated()'),self.handleCloseAll)
        
        self.closeActGrp.setEnabled(0)
        
        self.saveActGrp = QActionGroup(self)

        self.saveAct = QAction(self.trUtf8('Save'),
                QIconSet(IconFileSave),
                self.trUtf8('&Save'),
                QKeySequence(self.trUtf8("CTRL+S","File|Save")),
                self.saveActGrp)
        self.saveAct.setStatusTip(self.trUtf8('Save the current file'))
        self.saveAct.setWhatsThis(self.trUtf8(
            """<b>Save File</b>"""
            """<p>Save the contents of current editor window.</p>"""
        ))
        self.connect(self.saveAct,SIGNAL('activated()'),self.saveCurrentEditor)

        self.saveAsAct = QAction(self.trUtf8('Save as'),
                QIconSet(IconFileSaveAs),
                self.trUtf8('Save &as...'),
                QKeySequence(self.trUtf8("SHIFT+CTRL+S","File|Save As")),
                self.saveActGrp)
        self.saveAsAct.setStatusTip(self.trUtf8('Save the current file to a new one'))
        self.saveAsAct.setWhatsThis(self.trUtf8(
            """<b>Save File as</b>"""
            """<p>Save the contents of current editor window to a new file."""
            """ The file can be entered in a file selection dialog.</p>"""
        ))
        self.connect(self.saveAsAct,SIGNAL('activated()'),self.saveAsCurrentEditor)

        self.saveAllAct = QAction(self.trUtf8('Save all'),
                QIconSet(IconFileSaveAll),
                self.trUtf8('Save a&ll...'),
                0,self.saveActGrp)
        self.saveAllAct.setStatusTip(self.trUtf8('Save all files'))
        self.saveAllAct.setWhatsThis(self.trUtf8(
            """<b>Save All Files</b>"""
            """<p>Save the contents of all editor windows.</p>"""
        ))
        self.connect(self.saveAllAct,SIGNAL('activated()'),self.saveAllEditors)
        
        self.saveActGrp.setEnabled(0)

        self.saveToProjectAct = QAction(self.trUtf8('Save to Project'),
                QIconSet(IconFileSaveProject),
                self.trUtf8('Save to Pro&ject'),
                0,self)
        self.saveToProjectAct.setStatusTip(self.trUtf8('Save the current file to the current project'))
        self.saveToProjectAct.setWhatsThis(self.trUtf8(
            """<b>Save to Project</b>"""
            """<p>Save the contents of the current editor window to the"""
            """ current project. After the file has been saved, it is"""
            """ automatically added to the current project.</p>"""
        ))
        self.connect(self.saveToProjectAct,SIGNAL('activated()'),self.saveCurrentEditorToProject)
        self.saveToProjectAct.setEnabled(0)
        
        self.printAct = QAction(self.trUtf8('Print'),
                QIconSet(IconPrint),
                self.trUtf8('&Print'),
                QKeySequence(self.trUtf8("CTRL+P","File|Print")),self)
        self.printAct.setStatusTip(self.trUtf8('Print the current file'))
        self.printAct.setWhatsThis(self.trUtf8(
            """<b>Print File</b>"""
            """<p>Print the contents of current editor window.</p>"""
        ))
        self.connect(self.printAct,SIGNAL('activated()'),self.printCurrentEditor)
        self.printAct.setEnabled(0)

        self.printSelAct = QAction(self.trUtf8('Print Selection'),
                QIconSet(IconPrint),
                self.trUtf8('Prin&t Selection'),
                QKeySequence(self.trUtf8("SHIFT+CTRL+P","File|Print")),self)
        self.printSelAct.setStatusTip(self.trUtf8('Print the selection of the current file'))
        self.printSelAct.setWhatsThis(self.trUtf8(
            """<b>Print Selection</b>"""
            """<p>Print the selection of the current editor window.</p>"""
        ))
        self.connect(self.printSelAct,SIGNAL('activated()'),self.printCurrentEditorSel)
        self.printSelAct.setEnabled(0)

    def initFileMenu(self):
        """
        Public method to create the File menu.
        
        Returns
        
            the generated menu
        """
        menu = QPopupMenu(self.ui)
        self.recentMenu = QPopupMenu(menu)
        menu.insertTearOffHandle()
        self.newAct.addTo(menu)
        self.openAct.addTo(menu)
        menu.insertItem(self.trUtf8('Open &Recent Files'), self.recentMenu)
        self.connect(self.recentMenu,SIGNAL('aboutToShow()'),self.handleShowRecentMenu)
        menu.insertSeparator()
        self.closeAct.addTo(menu)
        self.closeAllAct.addTo(menu)
        menu.insertSeparator()
        self.saveAct.addTo(menu)
        self.saveAsAct.addTo(menu)
        self.saveAllAct.addTo(menu)
        self.saveToProjectAct.addTo(menu)
        menu.insertSeparator()
        self.printAct.addTo(menu)
        self.printSelAct.addTo(menu)

        return menu
        
    def initFileToolbar(self):
        """
        Public method to create the File toolbar.
        
        Returns
        
            the generated toolbar
        """
        tb = QToolBar(self.ui)
        self.newAct.addTo(tb)
        self.openAct.addTo(tb)
        self.closeAct.addTo(tb)
        self.saveAct.addTo(tb)
        self.saveAsAct.addTo(tb)
        self.saveAllAct.addTo(tb)
        self.saveToProjectAct.addTo(tb)
        tb.addSeparator()
        self.printAct.addTo(tb)
        
        return tb
        
    ##################################################################
    ## Initialize the edit related actions, edit menu and toolbar
    ##################################################################
    
    def initEditActions(self):
        """
        Private method defining the user interface actions for the edit commands.
        """
        self.editActGrp = QActionGroup(self)

        self.undoAct = QAction(self.trUtf8('Undo'),
                QIconSet(IconEditUndo),
                self.trUtf8('&Undo'),
                QKeySequence(self.trUtf8("CTRL+Z","Edit|Undo")),
                self.editActGrp)
        self.undoAct.setStatusTip(self.trUtf8('Undo the last change'))
        self.undoAct.setWhatsThis(self.trUtf8(
            """<b>Undo</b>"""
            """<p>Undo the last change done in the current editor.</p>"""
        ))
        self.connect(self.undoAct,SIGNAL('activated()'),self.handleEditUndo)
        
        self.redoAct = QAction(self.trUtf8('Redo'),
                QIconSet(IconEditRedo),
                self.trUtf8('&Redo'),
                QKeySequence(self.trUtf8("CTRL+Y","Edit|Redo")),
                self.editActGrp)
        self.redoAct.setStatusTip(self.trUtf8('Redo the last change'))
        self.redoAct.setWhatsThis(self.trUtf8(
            """<b>Redo</b>"""
            """<p>Redo the last change done in the current editor.</p>"""
        ))
        self.connect(self.redoAct,SIGNAL('activated()'),self.handleEditRedo)
        
        self.cutAct = QAction(self.trUtf8('Cut'),
                QIconSet(IconEditCut),
                self.trUtf8('Cu&t'),
                QKeySequence(self.trUtf8("CTRL+X","Edit|Cut")),
                self.editActGrp)
        self.cutAct.setStatusTip(self.trUtf8('Cut the selection'))
        self.cutAct.setWhatsThis(self.trUtf8(
            """<b>Cut</b>"""
            """<p>Cut the selected text of the current editor to the clipboard.</p>"""
        ))
        self.connect(self.cutAct,SIGNAL('activated()'),self.handleEditCut)
        
        self.copyAct = QAction(self.trUtf8('Copy'),
                QIconSet(IconEditCopy),
                self.trUtf8('&Copy'),
                QKeySequence(self.trUtf8("CTRL+C","Edit|Copy")),
                self.editActGrp)
        self.copyAct.setStatusTip(self.trUtf8('Copy the selection'))
        self.copyAct.setWhatsThis(self.trUtf8(
            """<b>Copy</b>"""
            """<p>Copy the selected text of the current editor to the clipboard.</p>"""
        ))
        self.connect(self.copyAct,SIGNAL('activated()'),self.handleEditCopy)
        
        self.pasteAct = QAction(self.trUtf8('Paste'),
                QIconSet(IconEditPaste),
                self.trUtf8('&Paste'),
                QKeySequence(self.trUtf8("CTRL+V","Edit|Paste")),
                self.editActGrp)
        self.pasteAct.setStatusTip(self.trUtf8('Paste the last cut/copied text'))
        self.pasteAct.setWhatsThis(self.trUtf8(
            """<b>Paste</b>"""
            """<p>Paste the last cut/copied text from the clipboard to"""
            """ the current editor.</p>"""
        ))
        self.connect(self.pasteAct,SIGNAL('activated()'),self.handleEditPaste)
        
        self.deleteAct = QAction(self.trUtf8('Clear'),
                QIconSet(IconEditDelete),
                self.trUtf8('Cl&ear'),
                0,self.editActGrp)
        self.deleteAct.setStatusTip(self.trUtf8('Clear all text'))
        self.deleteAct.setWhatsThis(self.trUtf8(
            """<b>Clear</b>"""
            """<p>Delete all text of the current editor.</p>"""
        ))
        self.connect(self.deleteAct,SIGNAL('activated()'),self.handleEditDelete)
        
        self.indentAct = QAction(self.trUtf8('Indent'),
                QIconSet(IconEditIndent),
                self.trUtf8('&Indent'),
                QKeySequence(self.trUtf8("CTRL+I","Edit|Indent")),
                self.editActGrp)
        self.indentAct.setStatusTip(self.trUtf8('Indent line'))
        self.indentAct.setWhatsThis(self.trUtf8(
            """<b>Indent</b>"""
            """<p>Indents the current line or the lines of the"""
            """ selection by one level.</p>"""
        ))
        self.connect(self.indentAct,SIGNAL('activated()'),self.handleEditIndent)
        
        self.unindentAct = QAction(self.trUtf8('Unindent'),
                QIconSet(IconEditUnindent),
                self.trUtf8('U&nindent'),
                QKeySequence(self.trUtf8("CTRL+U","Edit|Unindent")),
                self.editActGrp)
        self.unindentAct.setStatusTip(self.trUtf8('Unindent line'))
        self.unindentAct.setWhatsThis(self.trUtf8(
            """<b>Unindent</b>"""
            """<p>Unindents the current line or the lines of the"""
            """ selection by one level.</p>"""
        ))
        self.connect(self.unindentAct,SIGNAL('activated()'),self.handleEditUnindent)
        
        self.commentAct = QAction(self.trUtf8('Comment'),
                QIconSet(IconEditComment),
                self.trUtf8('C&omment'),
                QKeySequence(self.trUtf8("CTRL+M","Edit|Comment")),
                self.editActGrp)
        self.commentAct.setStatusTip(self.trUtf8('Comment Line or Selection'))
        self.commentAct.setWhatsThis(self.trUtf8(
            """<b>Comment</b>"""
            """<p>Comments the current line or the lines of the"""
            """ current selection.</p>"""
        ))
        self.connect(self.commentAct,SIGNAL('activated()'),self.handleEditComment)
        
        self.uncommentAct = QAction(self.trUtf8('Uncomment'),
                QIconSet(IconEditUncomment),
                self.trUtf8('Unco&mment'),
                QKeySequence(self.trUtf8("ALT+CTRL+M","Edit|Uncomment")),
                self.editActGrp)
        self.uncommentAct.setStatusTip(self.trUtf8('Uncomment Line or Selection'))
        self.uncommentAct.setWhatsThis(self.trUtf8(
            """<b>Uncomment</b>"""
            """<p>Uncomments the current line or the lines of the"""
            """ current selection.</p>"""
        ))
        self.connect(self.uncommentAct,SIGNAL('activated()'),self.handleEditUncomment)
        
        self.selectBraceAct = QAction(self.trUtf8('Select to brace'),
                self.trUtf8('Select to &brace'),
                QKeySequence(self.trUtf8("CTRL+E","Edit|Select to brace")),
                self.editActGrp)
        self.selectBraceAct.setStatusTip(self.trUtf8('Select text to the matching brace'))
        self.selectBraceAct.setWhatsThis(self.trUtf8(
            """<b>Selct to brace</b>"""
            """<p>Select text of the current editor to the matching brace.</p>"""
        ))
        self.connect(self.selectBraceAct,SIGNAL('activated()'),self.handleEditSelectBrace)
        
        self.selectAllAct = QAction(self.trUtf8('Select all'),
                self.trUtf8('&Select all'),
                QKeySequence(self.trUtf8("CTRL+A","Edit|Select all")),
                self.editActGrp)
        self.selectAllAct.setStatusTip(self.trUtf8('Select all text'))
        self.selectAllAct.setWhatsThis(self.trUtf8(
            """<b>Selct All</b>"""
            """<p>Select all text of the current editor.</p>"""
        ))
        self.connect(self.selectAllAct,SIGNAL('activated()'),self.handleEditSelectAll)
        
        self.deselectAllAct = QAction(self.trUtf8('Deselect all'),
                self.trUtf8('&Deselect all'),
                QKeySequence(self.trUtf8("ALT+CTRL+A","Edit|Deselect all")),
                self.editActGrp)
        self.deselectAllAct.setStatusTip(self.trUtf8('Deselect all text'))
        self.deselectAllAct.setWhatsThis(self.trUtf8(
            """<b>Deselct All</b>"""
            """<p>Deselect all text of the current editor.</p>"""
        ))
        self.connect(self.deselectAllAct,SIGNAL('activated()'),self.handleEditDeselectAll)
        
        self.convertEOLAct = QAction(self.trUtf8('Convert Line End Characters'),
                self.trUtf8('Convert &Line End Characters'),
                0, self.editActGrp)
        self.convertEOLAct.setStatusTip(self.trUtf8('Convert Line End Characters'))
        self.convertEOLAct.setWhatsThis(self.trUtf8(
            """<b>Convert Line End Characters</b>"""
            """<p>Convert the line end characters to the currently set type.</p>"""
        ))
        self.connect(self.convertEOLAct,SIGNAL('activated()'),self.handleConvertEOL)
        
        self.editActGrp.setEnabled(0)
        
    def initEditMenu(self):
        """
        Public method to create the Edit menu
        
        Returns
        
            the generated menu
        """
        menu = QPopupMenu(self.ui)
        menu.insertTearOffHandle()
        self.undoAct.addTo(menu)
        self.redoAct.addTo(menu)
        menu.insertSeparator()
        self.cutAct.addTo(menu)
        self.copyAct.addTo(menu)
        self.pasteAct.addTo(menu)
        self.deleteAct.addTo(menu)
        menu.insertSeparator()
        self.indentAct.addTo(menu)
        self.unindentAct.addTo(menu)
        menu.insertSeparator()
        self.commentAct.addTo(menu)
        self.uncommentAct.addTo(menu)
        menu.insertSeparator()
        self.selectBraceAct.addTo(menu)
        self.selectAllAct.addTo(menu)
        self.deselectAllAct.addTo(menu)
        menu.insertSeparator()
        self.convertEOLAct.addTo(menu)
        
        return menu
        
    def initEditToolbar(self):
        """
        Public method to create the Edit toolbar
        
        Returns
        
            the generated toolbar
        """
        tb = QToolBar(self.ui)
        self.undoAct.addTo(tb)
        self.redoAct.addTo(tb)
        tb.addSeparator()
        self.cutAct.addTo(tb)
        self.copyAct.addTo(tb)
        self.pasteAct.addTo(tb)
        self.deleteAct.addTo(tb)
        tb.addSeparator()
        self.indentAct.addTo(tb)
        self.unindentAct.addTo(tb)
        tb.addSeparator()
        self.commentAct.addTo(tb)
        self.uncommentAct.addTo(tb)
        
        return tb
        
    ##################################################################
    ## Initialize the search related actions, search menu and toolbar
    ##################################################################
    
    def initSearchActions(self):
        """
        Private method defining the user interface actions for the search commands.
        """
        self.searchActGrp = QActionGroup(self)
        
        self.searchAct = QAction(self.trUtf8('Search'),
                QIconSet(IconFind),
                self.trUtf8('&Search...'),
                QKeySequence(self.trUtf8("CTRL+F","Search|Search")),
                self.searchActGrp)
        self.searchAct.setStatusTip(self.trUtf8('Search for a text'))
        self.searchAct.setWhatsThis(self.trUtf8(
            """<b>Search</b>"""
            """<p>Search for some text in the current editor. A"""
            """ dialog is shown to enter the searchtext and options"""
            """ for the search.</p>"""
        ))
        self.connect(self.searchAct,SIGNAL('activated()'),self.handleSearch)
        
        self.searchAgainAct = QAction(self.trUtf8('Search again'),
                QIconSet(IconFindNext),
                self.trUtf8('Search &again'),
                Qt.Key_F3,self.searchActGrp)
        self.searchAgainAct.setStatusTip(self.trUtf8('Search again for text'))
        self.searchAgainAct.setWhatsThis(self.trUtf8(
            """<b>Search again</b>"""
            """<p>Search again for some text in the current editor."""
            """ The previously entered searchtext and options are reused.</p>"""
        ))
        self.connect(self.searchAgainAct,SIGNAL('activated()'),self.searchDlg.handleFindNext)
        
        self.replaceAct = QAction(self.trUtf8('Replace'),
                self.trUtf8('&Replace...'),
                QKeySequence(self.trUtf8("CTRL+R","Search|Replace")),
                self.searchActGrp)
        self.replaceAct.setStatusTip(self.trUtf8('Replace some text'))
        self.replaceAct.setWhatsThis(self.trUtf8(
            """<b>Replace</b>"""
            """<p>Search for some text in the current editor and replace it. A"""
            """ dialog is shown to enter the searchtext, the replacement text"""
            """ and options for the search and replace.</p>"""
        ))
        self.connect(self.replaceAct,SIGNAL('activated()'),self.handleReplace)
        
        self.gotoAct = QAction(self.trUtf8('Goto Line'),
                QIconSet(IconGoto),
                self.trUtf8('&Goto Line...'),
                QKeySequence(self.trUtf8("CTRL+G","Search|Goto Line")),
                self.searchActGrp)
        self.gotoAct.setStatusTip(self.trUtf8('Goto Line'))
        self.gotoAct.setWhatsThis(self.trUtf8(
            """<b>Goto Line</b>"""
            """<p>Go to a specific line of text in the current editor."""
            """ A dialog is shown to enter the linenumber.</p>"""
        ))
        self.connect(self.gotoAct,SIGNAL('activated()'),self.handleGoto)
        
        self.gotoBraceAct = QAction(self.trUtf8('Goto Brace'),
                QIconSet(IconGotoBrace),
                self.trUtf8('Goto &Brace'),
                QKeySequence(self.trUtf8("CTRL+L","Search|Goto Brace")),
                self.searchActGrp)
        self.gotoBraceAct.setStatusTip(self.trUtf8('Goto Brace'))
        self.gotoBraceAct.setWhatsThis(self.trUtf8(
            """<b>Goto Brace</b>"""
            """<p>Go to the matching brace in the current editor.</p>"""
        ))
        self.connect(self.gotoBraceAct,SIGNAL('activated()'),self.handleGotoBrace)
        
        self.searchActGrp.setEnabled(0)
        
        self.searchFilesAct = QAction(self.trUtf8('Search in Files'),
                QIconSet(IconProjectFind),
                self.trUtf8('Search in &Files...'),
                QKeySequence(self.trUtf8("SHIFT+CTRL+F","Search|Search Files")),
                self)
        self.searchFilesAct.setStatusTip(self.trUtf8('Search for a text in files'))
        self.searchFilesAct.setWhatsThis(self.trUtf8(
            """<b>Search in Files</b>"""
            """<p>Search for some text in the files of a directory tree"""
            """ or the project. A dialog is shown to enter the searchtext"""
            """ and options for the search and to display the result.</p>"""
        ))
        self.connect(self.searchFilesAct,SIGNAL('activated()'),self.handleSearchFiles)
        
    def initSearchMenu(self):
        """
        Public method to create the Search menu
        
        Returns
        
            the generated menu
        """
        menu = QPopupMenu(self.ui)
        menu.insertTearOffHandle()
        self.searchAct.addTo(menu)
        self.searchAgainAct.addTo(menu)
        self.replaceAct.addTo(menu)
        menu.insertSeparator()
        self.searchFilesAct.addTo(menu)
        menu.insertSeparator()
        self.gotoAct.addTo(menu)
        self.gotoBraceAct.addTo(menu)
        
        return menu
        
    def initSearchToolbar(self):
        """
        Public method to create the Search toolbar
        
        Returns
        
            the generated toolbar
        """
        tb = QToolBar(self.ui)
        self.searchAct.addTo(tb)
        self.searchAgainAct.addTo(tb)
        tb.addSeparator()
        self.searchFilesAct.addTo(tb)
        tb.addSeparator()
        self.gotoAct.addTo(tb)
        self.gotoBraceAct.addTo(tb)
        
        return tb
        
    ##################################################################
    ## Initialize the view related actions, view menu and toolbar
    ##################################################################
    
    def initViewActions(self):
        """
        Private method defining the user interface actions for the view commands.
        """
        self.viewActGrp = QActionGroup(self)
        self.viewFoldActGrp = QActionGroup(self)

        self.zoomInAct = QAction(self.trUtf8('Zoom in'),
                            QIconSet(IconZoomIn),
                            self.trUtf8('Zoom &in'),
                            Qt.CTRL+Qt.Key_Plus, self.viewActGrp)
        self.zoomInAct.setStatusTip(self.trUtf8('Zoom in on the text'))
        self.zoomInAct.setWhatsThis(self.trUtf8(
                """<b>Zoom in</b>"""
                """<p>Zoom in on the text. This makes the text bigger.</p>"""
                ))
        self.connect(self.zoomInAct,SIGNAL('activated()'),self.handleZoomIn)
        
        self.zoomOutAct = QAction(self.trUtf8('Zoom out'),
                            QIconSet(IconZoomOut),
                            self.trUtf8('Zoom &out'),
                            Qt.CTRL+Qt.Key_Minus, self.viewActGrp)
        self.zoomOutAct.setStatusTip(self.trUtf8('Zoom out on the text'))
        self.zoomOutAct.setWhatsThis(self.trUtf8(
                """<b>Zoom out</b>"""
                """<p>Zoom out on the text. This makes the text smaller.</p>"""
                ))
        self.connect(self.zoomOutAct,SIGNAL('activated()'),self.handleZoomOut)
        
        self.zoomToAct = QAction(self.trUtf8('Zoom'),
                            QIconSet(IconZoomTo),
                            self.trUtf8('&Zoom'),
                            0, self.viewActGrp)
        self.zoomToAct.setStatusTip(self.trUtf8('Zoom the text'))
        self.zoomToAct.setWhatsThis(self.trUtf8(
                """<b>Zoom</b>"""
                """<p>Zoom the text. This opens a dialog where the"""
                """ desired size can be entered.</p>"""
                ))
        self.connect(self.zoomToAct,SIGNAL('activated()'),self.handleZoom)
        
        self.toggleAllAct = QAction(self.trUtf8('Toggle all folds'),
                            self.trUtf8('Toggle &all folds'),
                            0, self.viewFoldActGrp)
        self.toggleAllAct.setStatusTip(self.trUtf8('Toggle all folds'))
        self.toggleAllAct.setWhatsThis(self.trUtf8(
                """<b>Toggle all folds</b>"""
                """<p>Toggle all folds of the current editor.</p>"""
                ))
        self.connect(self.toggleAllAct,SIGNAL('activated()'),self.handleToggleAll)
        
        self.toggleCurrentAct = QAction(self.trUtf8('Toggle current fold'),
                            self.trUtf8('Toggle &current fold'),
                            0, self.viewFoldActGrp)
        self.toggleCurrentAct.setStatusTip(self.trUtf8('Toggle current fold'))
        self.toggleCurrentAct.setWhatsThis(self.trUtf8(
                """<b>Toggle current fold</b>"""
                """<p>Toggle the folds of the current line of the current editor.</p>"""
                ))
        self.connect(self.toggleCurrentAct,SIGNAL('activated()'),self.handleToggleCurrent)
        
        self.viewActGrp.setEnabled(0)
        self.viewFoldActGrp.setEnabled(0)
        
    def initViewMenu(self):
        """
        Public method to create the View menu
        
        Returns
        
            the generated menu
        """
        menu = QPopupMenu(self.ui)
        menu.insertTearOffHandle()
        self.viewActGrp.addTo(menu)
        menu.insertSeparator()
        self.viewFoldActGrp.addTo(menu)
        
        return menu
        
    def initViewToolbar(self):
        """
        Public method to create the View toolbar
        
        Returns
        
            the generated toolbar
        """
        tb = QToolBar(self.ui)
        self.viewActGrp.addTo(tb)
        
        return tb
        
    ##################################################################
    ## Methods and slots that deal with file and window handling
    ##################################################################
    
    def handleOpen(self,prog=None):
        """
        Public slot to open a Python (or other) file.
        
        Arguments
        
            prog -- name of file to be opened (string or QString)
        """
        # Get the file name if one wasn't specified.
        if prog is None:
            prog = QFileDialog.getOpenFileName(None,
                        self.trUtf8('Python Files (*.py);;'
                            'C Files (*.h *.c);;'
                            'C++ Files (*.h *.hpp *.cxx *.cpp);;'
                            'C# Files (*.cs);;'
                            'IDL Files (*.idl);;'
                            'All Files (*)'),
                        self.ui)

            if prog.isNull():
                return

        prog = Utilities.normabspath(str(prog))

        # Open up the new file.
        self.handlePythonFile(prog)

    def checkDirty(self, editor):
        """
        Private method to check dirty status and open a message window.
        
        Arguments
        
            editor -- editor window to check
            
        Returns
        
            flag indicating successful reset of the dirty flag (boolean)
        """
        if editor.isModified():
            fn = editor.getFileName()
            if fn is None:
                fn = self.trUtf8('Noname')
            res = QMessageBox.warning(self.parent(), 
                self.trUtf8("File Modified"),
                self.trUtf8("The file <b>%1</b> has unsaved changes.")
                    .arg(fn),
                self.trUtf8("&Save"), self.trUtf8("&Discard changes"),
                self.trUtf8("&Abort"), 0, 2)
            if res == 0:
                (ok, newName) = editor.saveFile()
                if ok:
                    self.setEditorName(editor, newName)
                return ok
            elif res == 2:
                return 0
        
        return 1
        
    def closeEditor(self, editor):
        """
        Private method to close an editor window.
        
        Arguments
        
            editor -- editor window to be closed
            
        Returns
        
            flag indicating success (boolean)
        """
        # save file if necessary
        if not self.checkDirty(editor):
            return 0
            
        # remove all breakpoints of the editor
        self.dbs.clearFileBreakpoints(editor.getFileName())
        
        # remove the window
        self.removeView(editor)
        self.editors.remove(editor)
        if not len(self.editors):
            self.handleLastEditorClosed()
            self.emit(PYSIGNAL('lastEditorClosed'), ())
        return 1
    
    def handleClose(self):
        """
        Public method to close the current window.
        
        Returns
        
            flag indicating success (boolean)
        """
        aw = self.activeWindow()
        if aw is None:
            return 0
            
        return self.closeEditor(aw)
            
    def handleCloseAll(self):
        """
        Private method to close all editor windows via file menu.
        """
        savedEditors = self.editors[:]
        for editor in savedEditors:
            self.closeEditor(editor)
            
    def handleCloseWindow(self, fn):
        """
        Public method to close an arbitrary source editor.
        
        Arguments
        
            fn -- filename of editor to be closed
            
        Returns
        
            flag indicating success (boolean)
        """
        for editor in self.editors:
            if Utilities.samepath(fn, editor.getFileName()):
                break
        else:
            return 1
            
        return self.closeEditor(editor)
        
    def handleExit(self):
        """
        Public method to handle the debugged program terminating.
        """
        if self.currentEditor is not None:
            self.currentEditor.highlight()
            self.currentEditor = None
            
        self.setSbFile()

    def handlePythonFile(self,pyfn,lineno=None):
        """
        Public method to handle the user selecting a file for display.
        
        Arguments
        
            pyfn -- name of file to be opened
            
            lineno -- line number to place the cursor at
        """
        try:
            self.displayPythonFile(pyfn,lineno)
        except IOError:
            pass

    def displayPythonFile(self,fn,lineno=None):
        """
        Public slot to display a file in an editor.
        
        Arguments
        
            fn -- name of file to be opened
            
            lineno -- line number to place the cursor at
        """
        newWin, editor = self.getEditor(fn)
        
        if newWin:
            self.addView(editor, fn)
            self.handleModificationStatusChanged(editor.isModified(), editor)
        else:
            self.showView(editor, fn)
        self.checkActions(editor)
            
        if lineno is not None:
            editor.ensureVisible(lineno-1)
            editor.gotoLine(lineno-1)
        
        # insert filename into list of recently opened projects
        self.recent.remove(fn)
        self.recent.prepend(fn)
        if len(self.recent) > 9:
            self.recent = self.recent[:9]

    def toggleWindow(self,w):
        """
        Private method to toggle a workspace window.
        
        Arguments
        
            w -- editor window to be toggled
        """
        if w.isHidden():
            w.show()
        else:
            w.hide()

    def setFileLine(self,fn,line,error=0):
        """
        Public method to update the user interface when the current program or line changes.
        
        Arguments
        
            fn -- filename of editor to update (string)
            
            line -- line number to highlight (int)
            
            error -- flag indicating an error highlight (boolean)
        """
        self.setSbFile(fn,line)

        try:
            newWin, self.currentEditor = self.getEditor(fn)
        except IOError:
            return

        # Change the highlighted line.
        self.currentEditor.highlight(line,error)

        if newWin:
            self.addView(self.currentEditor, fn)
        else:
            self.showView(self.currentEditor, fn)
        self.currentEditor.highlightVisible()
        self.checkActions(self.currentEditor, 0)
            
    def setSbFile(self,fn=None,line=None,pos=None):
        """
        Private method to set the file info in the status bar.
        
        Arguments
        
            fn -- filename to display (string)
            
            line -- line number to display (int)
            
            pos -- character position to display (int)
        """
        if fn is None:
            fn = ''
            writ = '   '
        else:
            if QFileInfo(fn).isWritable():
                writ = ' rw'
            else:
                writ = ' ro'
        
        self.sbWritable.setText(writ)
        self.sbFile.setText(self.trUtf8('File: %1').arg(fn,-50))

        if line is None:
            line = ''

        self.sbLine.setText(self.trUtf8('Line: %1').arg(line,5))

        if pos is None:
            pos = ''
            
        self.sbPos.setText(self.trUtf8('Pos: %1').arg(pos, 5))
        
    def unhighlight(self, current=0):
        """
        Public method to switch off all highlights.
        
        Arguments
        
            current -- flag indicating only the current editor should be unhighlighted
                (boolean)
        """
        if current and self.currentEditor is not None:
            self.currentEditor.highlight()
        else:
            for editor in self.editors:
                editor.highlight()

    def getOpenFilenames(self):
        """
        Public method returning a list of the filenames of all editors.
        
        Returns
        
            list of all opened filenames (list of strings)
        """
        filenames = []
        for editor in self.editors:
            fn = editor.getFileName()
            if fn is not None:
                filenames.append(fn)
                
        return filenames
        
    def getEditor(self,fn):
        """
        Private method to return the editor displaying the given file.
        
        If there is no editor with the given file, a new editor window is
        created.
        
        Arguments
        
            fn -- filename to look for
            
        Returns
        
            tuple of two values giving a flag indicating a new window creation and
            a reference to the editor displaying this file
        """
        newWin = 0
        for editor in self.editors:
            if Utilities.samepath(fn, editor.getFileName()):
                break
        else:
            editor = Editor(self.dbs,fn,self)
            self.editors.append(editor)
            self.connect(editor, PYSIGNAL('modificationStatusChanged'),
                self.handleModificationStatusChanged)
            self.connect(editor, PYSIGNAL('cursorChanged'), self.setSbFile)
            self.connect(editor, PYSIGNAL('editorSaved'), self.handleEditorSaved)
            self.handleEditorOpened()
            self.emit(PYSIGNAL('editorOpened'), ())
            newWin = 1

        return (newWin, editor)

    def getActiveName(self):
        """
        Public method to retrieve the filename of the active window.
        
        Returns
        
            filename of active window (string)
        """
        aw = self.activeWindow()
        if aw:
            return aw.getFileName()
        else:
            return None

    def handleBreakpoint(self, fn, line, state = Qt.LeftButton, cond = None):
        """
        Public method to set or reset a breakpoint programatically.
        
        Arguments
        
            fn -- filename of the breakpoint operation (string)
            
            line -- line number of the breakpoint operation (int)
            
            state -- mouse button state (combination of Qt.LeftButton
                and Qt.ShiftButton)
                
            cond -- condition for the breakpoint (string or QString)
        """
        try:
            dummy, editor = self.getEditor(fn)
        except IOError:
            return
        # we simulate a button press in the breakpoint column 
        # of the source window
        editor.handleBreakpoint(line, state, cond)
        
    def saveEditor(self, fn):
        """
        Public method to save a named editor file.
        
        Arguments
        
            fn -- filename of editor to be saved (string)
            
        Returns
        
            flag indicating success (boolean)
        """
        for editor in self.editors:
            if Utilities.samepath(fn, editor.getFileName()):
                break
        else:
            return 1
            
        if not editor.isModified():
            return 1
        else:
            ok, dummy = editor.saveFile()
            return ok
        
    def saveCurrentEditor(self):
        """
        Public slot to save the contents of the current editor.
        """
        aw = self.activeWindow()
        if aw:
            ok, newName = aw.saveFile()
            if ok:
                self.setEditorName(aw, newName)
        else:
            return

    def saveAsCurrentEditor(self):
        """
        Public slot to save the contents of the current editor to a new file.
        """
        aw = self.activeWindow()
        if aw:
            ok, newName = aw.saveFileAs()
            if ok:
                self.setEditorName(aw, newName)
        else:
            return

    def saveAllEditors(self):
        """
        Public slot to save the contents of all editors.
        """
        for editor in self.editors:
            ok, newName = editor.saveFile()
            if ok:
                self.setEditorName(editor, newName)

    def saveCurrentEditorToProject(self):
        """
        Public slot to save the contents of the current editor to the current project.
        """
        pro = self.ui.getProject()
        path = pro.ppath
        aw = self.activeWindow()
        if aw:
            ok, newName = aw.saveFileAs(path)
            if ok:
                self.setEditorName(aw, newName)
                pro.appendFile(newName)
        else:
            return
        
    def newEditor(self):
        """
        Public slot to generate a new empty editor.
        """
        editor = Editor(self.dbs, None, self)
        self.editors.append(editor)
        self.connect(editor, PYSIGNAL('modificationStatusChanged'),
            self.handleModificationStatusChanged)
        self.connect(editor, PYSIGNAL('cursorChanged'), self.setSbFile)
        self.connect(editor, PYSIGNAL('editorSaved'), self.handleEditorSaved)
        self.addView(editor, None)
        self.handleEditorOpened()
        self.checkActions(editor)
        self.emit(PYSIGNAL('editorOpened'), ())
        
    def printCurrentEditor(self):
        """
        Public slot to print the contents of the current editor.
        """
        aw = self.activeWindow()
        if aw:
            aw.printFile()
        else:
            return

    def printCurrentEditorSel(self):
        """
        Public slot to print the selection of the current editor.
        """
        aw = self.activeWindow()
        if aw:
            aw.printSelection()
        else:
            return

    def handleShowRecentMenu(self):
        """
        Private method to set up recent files menu.
        """
        idx = 0
        self.recentMenu.clear()
        
        for rp in self.recent:
            id = self.recentMenu.insertItem('&%d. %s' % (idx+1, str(rp)),
                                            self.handleOpenRecent)
            self.recentMenu.setItemParameter(id,idx)
            
            idx = idx + 1
            
        self.recentMenu.insertSeparator()
        self.recentMenu.insertItem(self.trUtf8('Clear'), self.handleClearRecent)
        
    def handleOpenRecent(self, idx):
        """
        Private method to open a file from the list of rencently opened files.
        
        Arguments
        
            idx -- index of the selected entry (int)
        """
        self.handlePythonFile(str(self.recent[idx]))
        
    def handleClearRecent(self):
        """
        Private method to clear the recent files menu.
        """
        self.recent = QStringList()
        
    def handleNewProject(self):
        """
        Public slot to handle the NewProject signal.
        """
        self.saveToProjectAct.setEnabled(1)
        
    def handleProjectOpened(self):
        """
        Public slot to handle the projectOpened signal.
        """
        self.saveToProjectAct.setEnabled(1)
        
    def handleProjectClosed(self):
        """
        Public slot to handle the projectClosed signal.
        """
        self.saveToProjectAct.setEnabled(0)

    ##################################################################
    ## Below are the action methods for the edit menu
    ##################################################################
    
    def handleEditUndo(self):
        """
        Private method to handle the undo action.
        """
        self.activeWindow().undo()
        
    def handleEditRedo(self):
        """
        Private method to handle the redo action.
        """
        self.activeWindow().redo()
        
    def handleEditCut(self):
        """
        Private method to handle the cut action.
        """
        self.activeWindow().cut()
        
    def handleEditCopy(self):
        """
        Private method to handle the copy action.
        """
        if qApp.focusWidget() == self.ui.shell:
            self.ui.shell.copy()
        else:
            self.activeWindow().copy()
        
    def handleEditPaste(self):
        """
        Private method to handle the paste action.
        """
        if qApp.focusWidget() == self.ui.shell:
            self.ui.shell.paste()
        else:
            self.activeWindow().paste()
        
    def handleEditDelete(self):
        """
        Private method to handle the delete action.
        """
        self.activeWindow().clear()
        
    def handleEditIndent(self):
        """
        Private method to handle the indent action.
        """
        self.activeWindow().indentLineOrSelection()
        
    def handleEditUnindent(self):
        """
        Private method to handle the unindent action.
        """
        self.activeWindow().unindentLineOrSelection()
        
    def handleEditComment(self):
        """
        Private method to handle the comment action.
        """
        self.activeWindow().commentLineOrSelection()
        
    def handleEditUncomment(self):
        """
        Private method to handle the uncomment action.
        """
        self.activeWindow().uncommentLineOrSelection()
        
    def handleEditSelectBrace(self):
        """
        Private method to handle the select to brace action.
        """
        self.activeWindow().selectToMatchingBrace()
        
    def handleEditSelectAll(self):
        """
        Private method to handle the select all action.
        """
        self.activeWindow().selectAll(1)
        
    def handleEditDeselectAll(self):
        """
        Private method to handle the select all action.
        """
        self.activeWindow().selectAll(0)
        
    def handleConvertEOL(self):
        """
        Private method to handle the convert line end characters action.
        """
        aw = self.activeWindow()
        aw.convertEols(aw.eolMode())
        
    ##################################################################
    ## Below are the action and utility methods for the search menu
    ##################################################################

    def getWord(self, text, index):
        """
        Private method to get the word at a position.
        
        Arguments
        
            text -- text to look at (string or QString)
            
            index -- position to look at (int)
            
        Returns
        
            the word at that position
        """
        re = QRegExp('[^\w_]')
        start = text.findRev(re, index) + 1
        end = text.find(re, index)
        if end > start:
            word = text.mid(start, end-start)
        else:
            word = QString('')
        return word
        
    def textForFind(self):
        """
        Private method to determine the selection or the current word for the next find operation.
        
        Returns
        
            selection or current word (QString)
        """
        aw = self.activeWindow()
        if aw.hasSelectedText():
            text = aw.selectedText()
            if text.contains('\r') or text.contains('\n'):
                # the selection contains at least a newline, it is
                # unlikely to be the expression to search for
                return ''
                
            return text
            
        # no selected text, determine the word at the current position
        line, index = aw.getCursorPosition()
        return self.getWord(aw.text(line), index)
        
    def getSRHistory(self, key):
        """
        Private method to get the search or replace history list.
        
        Arguments
        
            key -- list to return (must be 'search' or 'replace')
            
        Returns
        
            the requested history list (QStringList)
        """
        return self.srHistory[key]
        
    def handleSearch(self):
        """
        Private method to handle the search action.
        """
        self.searchDlg.showFind(self.textForFind())
        
    def handleReplace(self):
        """
        Private method to handle the replace action.
        """
        self.replaceDlg.showReplace(self.textForFind())
        
    def handleGoto(self):
        """
        Private method to handle the goto action.
        """
        aw = self.activeWindow()
        dlg = GotoDialog(self.ui, None, 1)
        dlg.setMaxValue(aw.lines())
        if dlg.exec_loop() == QDialog.Accepted:
            aw.gotoLine(dlg.getLinenumber()-1)
        
    def handleGotoBrace(self):
        """
        Private method to handle the goto brace action.
        """
        self.activeWindow().moveToMatchingBrace()
        
    def handleSearchFiles(self):
        """
        Private method to handle the search in files action.
        """
        self.ui.findFilesDialog.show(self.textForFind())
        
    ##################################################################
    ## Below are the action methods for the view menu
    ##################################################################
    
    def handleZoomIn(self):
        """
        Private method to handle the zoom in action.
        """
        self.activeWindow().zoomIn()
        
    def handleZoomOut(self):
        """
        Private method to handle the zoom out action.
        """
        self.activeWindow().zoomOut()
        
    def handleZoom(self):
        """
        Private method to handle the zoom action.
        """
        dlg = ZoomDialog(self.ui, None, 1)
        if dlg.exec_loop() == QDialog.Accepted:
            self.activeWindow().zoomTo(dlg.getZoomSize())
            
    def handleToggleAll(self):
        """
        Private method to handle the toggle all folds action.
        """
        self.activeWindow().foldAll()
        
    def handleToggleCurrent(self):
        """
        Private method to handle the toggle current fold action.
        """
        aw = self.activeWindow()
        line, index = aw.getCursorPosition()
        aw.foldLine(line)
        
    ##################################################################
    ## Below are general utility methods
    ##################################################################
    
    def handleResetUI(self):
        """
        Public slot to handle the resetUI signal.
        """
        editor = self.activeWindow()
        if editor is None:
            self.setSbFile()
        else:
            line, pos = editor.getCursorPosition()
            self.setSbFile(editor.getFileName(), line+1, pos)
        
    def closeViewManager(self):
        """
        Public method to shutdown the viewmanager. 
        
        If it cannot close all editor windows, it aborts the shutdown process.
        
        Returns
        
            flag indicating success (boolean)
        """
        self.handleCloseAll()
            
        # save the list of recently opened projects
        ok = Preferences.Prefs.settings.writeEntry('/eric3/Recent/Sources', self.recent)
        
        if len(self.editors):
            return 0
        else:
            return 1

    def handleLastEditorClosed(self):
        """
        Private slot to handle the lastEditorClosed signal.
        """
        self.closeActGrp.setEnabled(0)
        self.saveActGrp.setEnabled(0)
        self.printAct.setEnabled(0)
        self.printSelAct.setEnabled(0)
        self.editActGrp.setEnabled(0)
        self.searchActGrp.setEnabled(0)
        self.viewActGrp.setEnabled(0)
        self.viewFoldActGrp.setEnabled(0)
        
    def handleEditorOpened(self):
        """
        Private slot to handle the editorOpened signal.
        """
        self.closeActGrp.setEnabled(1)
        self.saveActGrp.setEnabled(1)
        self.printAct.setEnabled(1)
        self.printSelAct.setEnabled(1)
        self.editActGrp.setEnabled(1)
        self.searchActGrp.setEnabled(1)
        self.viewActGrp.setEnabled(1)
        self.viewFoldActGrp.setEnabled(1)
        
    def checkActions(self, editor, setSb=1):
        """
        Private slot to check some actions for their enable/disable status and set the statusbar info.
        
        Arguments
        
            editor -- editor window
            
            setSb -- flag indicating an update of the status bar is wanted (boolean)
        """
        if editor is not None:
            self.saveAct.setEnabled(editor.isModified())
            
            if setSb:
                line, pos = editor.getCursorPosition()
                self.setSbFile(editor.getFileName(), line+1, pos)
        
    def handlePreferencesChanged(self):
        """
        Public slot to handle the preferencesChanged signal.
        
        This method performs the following actions
        
            - reread the colours for the syntax highlighting
            - changes in viewmanager type are activated on an
              application restart.
        """
        for editor in self.editors:
            editor.readSettings()
            
    def handleEditorSaved(self, fn):
        """
        Public slot to handle the editorSaved signal.
        
        It simply reemits the signal.
        
        Arguments
        
            fn -- filename of the saved editor
        """
        self.emit(PYSIGNAL('editorSaved'), (fn,))
