# -*- coding: utf-8 -*-

# Copyright (c) 2003 Detlev Offenbach <detlev@die-offenbachs.de>
#

"""
Module implementing the highlevel wizard functions.
"""

import sys
import os
import glob

from qt import *

class Wizards(QObject):
    """
    Class implementing the highlevel wizard functions.
    """
    def __init__(self, loc, parent = None):
        """
        Constructor
        
        Arguments
        
            loc -- the localizer string as determined by the main script (string)
        """
        QObject.__init__(self, parent)
        self.registeredWizards = []
        self.registeredTranslators = []
        self.loc = loc
        self.init()
       
        
    def init(self):
        """
        Private method to register all the wizards.
        """
        ericDir = os.path.dirname(sys.argv[0])
        if ericDir == '':
            ericDir = '.'
        wizDir = os.path.abspath(os.path.join(ericDir, 'Wizards'))
        configs = glob.glob('%s%s*.e3w' % (wizDir, os.sep))
        configs.sort()
        
        # now loop over all wizard config files
        for config in configs:
            imp, cls, trans, ok = self.parseWizardConfig(config)
            if not ok:
                continue
                
            if imp is None:
                # None acts as separator
                self.registeredWizards.append(None)
                continue
                
            try:
                exec 'import %s' % imp
                exec 'wizard = %s.%s(self)' % (imp, cls)
                self.registeredWizards.append(wizard)
                if trans is not None:
                    qtrans = QTranslator(None)
                    dir, dummy = os.path.splitext('%s.%s' % (wizDir, imp))
                    dir = dir.replace('.', os.sep)
                    if qtrans.load(trans, dir):
                        self.registeredTranslators.append(qtrans)
                        qApp.installTranslator(qtrans)
            except:
                print 'Could not bind wizard %s.%s!' % (imp, cls)
                continue
        
    def parseWizardConfig(self, fn):
        """
        Private method used to parse a wizards config file.
        
        The files have to be installed in the
        'Wizards' subdirectory of the eric3 installation directory.
        The order of the files in the directory will determine the
        structure of the Wizards menu. The files have the following
        structure.
        
        Lines beginning with '#' are comments and will be ignored.
        Lines beginning with these keyword will be processed.
        IMPORT: This names the module to be imported.
        CLASS: This names the class
        TRANSLATIONS: This names the basename of the translation files.
        
        All keywords are separated by '=' from their respective values.
        
        IMPORT and CLASS are mandatory, TRANSLATIONS is optional.
        
        An IMPORT value of 'Separator' will yield a separator line in
        the Wizards menu.
        
        The value of the TRANSLATIONS key will be combined with the
        location string determined at startup time and be given
        an extension of '.qm' to yield the filename to be loaded
        into a QTranslator object.
        
        The method returns a tuple of four values, the value of the IMPORT
        key, the value of the CLASS key, the value of the TRANSLATIONS
        key and a status value.
        
        Arguments
        
            fn -- filename of the wizards config file
            
        Returns
        
            tuple of four 3 strings and a boolean (module to be imported,
            class to be imported, basename of the translator file, flag
            indicating success)
        """
        try:
            f = open(fn, "rb")
            lines = f.readlines()
            f.close()
        except:
            return (None, None, None, 0)
            
        imp = None
        cls = None
        trans = None
        
        for line in lines:
            if line.startswith('#'):
                continue
            
            kv = line.split('=')
            if len(kv) != 2:
                continue
            
            key = kv[0].strip()
            if key == 'IMPORT':
                imp = kv[1].strip()
                if imp == 'Separator':
                    return (None, None, None, 1)
            elif key == 'CLASS':
                cls = kv[1].strip()
            elif key == 'TRANSLATIONS' and self.loc is not None:
                trans = '%s%s' % (kv[1].strip(), self.loc)
                
        if imp is None or cls is None:
            return (None, None, None, 0)
        else:
            return (imp, cls, trans, 1)
        
    def initMenu(self):
        """
        Public slot to initialize the wizards menu.
        
        Returns
        
            the generated menu
        """
        menu = QPopupMenu(self.parent())
        menu.insertTearOffHandle()
        
        for wizard in self.registeredWizards:
            if wizard == None:
                menu.insertSeparator()
            else:
                action = wizard.getAction()
                action.addTo(menu)
                        
        return menu
        
    def getCurrentEditor(self):
        """
        Public method to get a reference to the current editor.
        
        Returns
        
            the current editor
        """
        vm = self.parent().getViewManager()
        return  vm.activeWindow() 
                    
